@@ -36,8 +36,8 @@ from . import                 base
 
 class ApplyFlirtXfmAction(base.Action):
     """The ``ApplyFlirtXfmAction`` class is an action which allows the user to
-    load a FLIRT transformation matrix and apply it to the currently selected
-    overlay, if it is an :class:`.Image` instance.
+    load a FLIRT transformation matrix (or other affine file) and apply it to
+    the currently selected overlay, if it is an :class:`.Image` instance.
 
 
     A :class:`FlirtFileDialog` is used to prompt the user to select a
@@ -87,14 +87,17 @@ class ApplyFlirtXfmAction(base.Action):
         overlayList = self.__overlayList
         overlay     = displayCtx.getSelectedOverlay()
 
-        matFile, refFile = promptForFlirtFiles(
+        affType, matFile, refFile = promptForFlirtFiles(
             self.__frame, overlay, overlayList, displayCtx)
 
-        if matFile is None or refFile is None:
+        if all((affType is None, matFile is None, refFile is None)):
             return
 
-        xform = calculateTransform(
-            overlay, overlayList, displayCtx, matFile, refFile)
+        if affType == 'flirt':
+            xform = calculateTransform(
+                overlay, overlayList, displayCtx, matFile, refFile)
+        elif affType == 'v2w':
+            xform = np.loadtxt(matFile)
 
         overlay.voxToWorldMat = xform
 
@@ -139,6 +142,18 @@ def promptForFlirtFiles(parent, overlay, overlayList, displayCtx, save=False):
     :arg overlayList: The :class:`.OverlayList` instance.
     :arg displayCtx:  The :class:`.DisplayContext` instance.
     :arg save:        Prompt the user to save a transformation matrix instead.
+
+    :returns: A tuple containing:
+
+              - The affine type  currently one of ``'flirt'``, indicating a
+                FLIRT matrix, or ``'v2w'``, indicating a "raw" voxel-to-world
+                matrix.
+              - The selected matrix file.
+              - The selected reference image file (``None`` if
+                ``affType is 'v2w'``)
+
+             If the user cancelled the dialog, all elements of this tuple will
+             be ``None``.
     """
 
     import wx
@@ -182,12 +197,10 @@ def promptForFlirtFiles(parent, overlay, overlayList, displayCtx, save=False):
     dlg.Fit()
     dlg.CentreOnParent()
 
-    # TODO A checkbox/dropdown box allowing
-    #      the user to load a 'raw' transform,
-    #      instead of a FLIRT one?
-
-    if dlg.ShowModal() == wx.ID_OK: return dlg.GetMatFile(), dlg.GetRefFile()
-    else:                           return None, None
+    if dlg.ShowModal() == wx.ID_OK:
+        return dlg.GetAffineType(), dlg.GetMatFile(), dlg.GetRefFile()
+    else:
+        return None, None, None
 
 
 def guessFlirtFiles(path):
@@ -280,9 +293,9 @@ def guessFlirtFiles(path):
 
 
 class FlirtFileDialog(wx.Dialog):
-    """The ``FlirtFileDialog`` class is a ``wx.Dialog`` which prompts the
-    user to select a FLIRT transformation matrix and reference image associated
-    with a source image.
+    """The ``FlirtFileDialog`` class is a ``wx.Dialog`` which prompts the user to
+    select a FLIRT (or other affine) transformation matrix and reference image
+    associated with a source image.
 
     The user can select a reference image either from a drop down box, or
     by selecting a file in the file system.
@@ -321,6 +334,7 @@ class FlirtFileDialog(wx.Dialog):
 
         wx.Dialog.__init__(self,
                            parent,
+                           title=strings.titles[self],
                            style=(wx.DEFAULT_DIALOG_STYLE | wx.STAY_ON_TOP))
 
         if refOpts     is None: refOpts     = []
@@ -333,11 +347,17 @@ class FlirtFileDialog(wx.Dialog):
         self.__matFile     = None
         self.__refFile     = None
         self.__save        = save
+        self.__affineTypes = ['flirt', 'v2w']
+
+        affTypeOpts = [strings.labels[self, 'affType', at]
+                       for at in self.__affineTypes]
 
         refOpts = list(refOpts) + [strings.labels[self, 'refChoiceSelectFile']]
 
         overlayName    = wx.StaticText(self, style=wx.ALIGN_CENTRE_HORIZONTAL)
         label          = wx.StaticText(self, style=wx.ALIGN_CENTRE_HORIZONTAL)
+        affType        = wx.Choice(self, choices=affTypeOpts)
+        affTypeLabel   = wx.StaticText(self)
         refChoiceLabel = wx.StaticText(self)
         matFileLabel   = wx.StaticText(self)
         refFileLabel   = wx.StaticText(self)
@@ -349,17 +369,20 @@ class FlirtFileDialog(wx.Dialog):
         okButton       = wx.Button(self, wx.ID_OK)
         cancelButton   = wx.Button(self, wx.ID_CANCEL)
 
-        self.__matFileText   = matFileText
-        self.__refFileText   = refFileText
-        self.__refFileLabel  = refFileLabel
-        self.__refFileButton = refFileButton
-        self.__refChoice     = refChoice
+        self.__matFileText    = matFileText
+        self.__refFileText    = refFileText
+        self.__refFileLabel   = refFileLabel
+        self.__refFileButton  = refFileButton
+        self.__refChoiceLabel = refChoiceLabel
+        self.__refChoice      = refChoice
+        self.__affType        = affType
 
         srcName = op.basename(srcFile)
         srcName = fslimage.removeExt(srcFile)
 
         overlayName   .SetLabel(strings.labels[self, 'source'].format(srcName))
-        refChoiceLabel.SetLabel(strings.labels[self, 'refFile'])
+        affTypeLabel  .SetLabel(strings.labels[self, 'affType'])
+        refChoiceLabel.SetLabel(strings.labels[self, 'refImage'])
         matFileLabel  .SetLabel(strings.labels[self, 'matFile'])
         refFileLabel  .SetLabel(strings.labels[self, 'refFile'])
         matFileButton .SetLabel(strings.labels[self, 'selectFile'])
@@ -379,7 +402,7 @@ class FlirtFileDialog(wx.Dialog):
             refFileText.SetInsertionPointEnd()
 
         sizer       = wx.BoxSizer(wx.VERTICAL)
-        widgetSizer = wx.FlexGridSizer(3, 3, 0, 0)
+        widgetSizer = wx.FlexGridSizer(4, 3, 0, 0)
         btnSizer    = wx.BoxSizer(wx.HORIZONTAL)
 
         widgetSizer.AddGrowableCol(1)
@@ -394,6 +417,10 @@ class FlirtFileDialog(wx.Dialog):
         sizer.Add(btnSizer,    flag=wx.EXPAND | wx.LEFT | wx.RIGHT, border=10)
         sizer.Add((1, 10),     flag=wx.EXPAND)
 
+        widgetSizer.Add(affTypeLabel, flag=wx.ALL,    border=3)
+        widgetSizer.Add(affType,      flag=wx.EXPAND, proportion=1)
+        widgetSizer.Add((-1, -1))
+
         if refOpts is not None:
             widgetSizer.Add(refChoiceLabel, flag=wx.ALL,    border=3)
             widgetSizer.Add(refChoice,      flag=wx.EXPAND, proportion=1)
@@ -426,6 +453,7 @@ class FlirtFileDialog(wx.Dialog):
         cancelButton .Bind(wx.EVT_BUTTON, self.__onCancelButton)
         matFileButton.Bind(wx.EVT_BUTTON, self.__onMatFileButton)
         refFileButton.Bind(wx.EVT_BUTTON, self.__onRefFileButton)
+        affType      .Bind(wx.EVT_CHOICE, self.__onAffType)
         refChoice    .Bind(wx.EVT_CHOICE, self.__onRefChoice)
 
         if len(refOpts) == 1:
@@ -435,6 +463,14 @@ class FlirtFileDialog(wx.Dialog):
             self.__onRefChoice(None)
 
 
+    def GetAffineType(self):
+        """Return the currently selected affine type - currently either
+        ``'flirt'`` (indicating a FSL FLIRT matrix file), or ``'v2w'``
+        (indicating a "raw" voxel-to-world affine).
+        """
+        return self.__affineTypes[self.__affType.GetSelection()]
+
+
     def GetMatFile(self):
         """Returns the current value of the matrix file as a string, or
         ``None``, if the file path is not valid.
@@ -476,6 +512,22 @@ class FlirtFileDialog(wx.Dialog):
         self.EndModal(wx.ID_CANCEL)
 
 
+    def __onAffType(self, ev):
+        """Called when the user changes the affine type. Enables/disables
+        widgets related to the reference image (as they are only used for
+        FLIRT affines).
+        """
+
+        affType = self.__affineTypes[self.__affType.GetSelection()]
+        isFlirt = affType == 'flirt'
+
+        self.__refChoice     .Enable(isFlirt)
+        self.__refChoiceLabel.Enable(isFlirt)
+        self.__refFileLabel  .Enable(isFlirt)
+        self.__refFileText   .Enable(isFlirt)
+        self.__refFileButton .Enable(isFlirt)
+
+
     def __onRefChoice(self, ev):
         """Called when the user changes the selection in the reference image
         drop down box. Enables/disables the reference image file selection
