@@ -27,14 +27,16 @@ import            functools
 
 import            wx
 
-import fsl.utils.async                    as async
-from   fsl.utils.platform import platform as fslplatform
-import fsleyes_props                      as props
-import fsleyes_widgets.utils.typedict     as td
-import fsleyes.strings                    as strings
-import fsleyes.colourmaps                 as fslcm
-import fsleyes.actions.loadcolourmap      as loadcmap
-import fsleyes.actions.loadvertexdata     as loadvdata
+import fsl.utils.idle                 as idle
+import fsleyes_props                  as props
+import fsleyes_widgets                as fwidgets
+import fsleyes_widgets.imagepanel     as imagepanel
+import fsleyes_widgets.utils.typedict as td
+import fsleyes.strings                as strings
+import fsleyes.colourmaps             as fslcm
+import fsleyes.controls.colourbar     as cbar
+import fsleyes.actions.loadcolourmap  as loadcmap
+import fsleyes.actions.loadvertexdata as loadvdata
 
 
 _PROPERTIES      = td.TypeDict()
@@ -146,26 +148,32 @@ def _initPropertyList_Display(threedee):
 
 
 def _initPropertyList_VolumeOpts(threedee):
-    return ['custom_volume',
-            'interpolation',
-            'custom_cmap',
-            'cmapResolution',
-            'interpolateCmaps',
-            'invert',
-            'invertClipping',
-            'linkLowRanges',
-            'linkHighRanges',
-            'displayRange',
-            'clippingRange',
-            'clipImage',
-            'custom_overrideDataRange']
+    plist = ['custom_volume',
+             'interpolation',
+             'custom_cmap',
+             'cmapResolution',
+             'gamma',
+             'interpolateCmaps',
+             'invert',
+             'invertClipping',
+             'linkLowRanges',
+             'linkHighRanges',
+             'displayRange',
+             'clippingRange',
+             'clipImage',
+             'custom_overrideDataRange']
+
+    if threedee:
+        plist.remove('clipImage')
+
+    return plist
 
 
 def _init3DPropertyList_VolumeOpts():
     return ['blendFactor',
-            'resolution',
+            'smoothing',
             'numSteps',
-            'dithering',
+            'resolution',
             'custom_clipPlanes']
 
 
@@ -173,7 +181,10 @@ def _initPropertyList_MaskOpts(threedee):
     return ['custom_volume',
             'colour',
             'invert',
-            'threshold']
+            'threshold',
+            'interpolation',
+            'outline',
+            'outlineWidth']
 
 
 def _initPropertyList_VectorOpts(threedee):
@@ -217,11 +228,13 @@ def _initPropertyList_MeshOpts(threedee):
              'outline',
              'outlineWidth',
              'colour',
+             'custom_vertexSet',
              'custom_vertexData',
              'vertexDataIndex',
              'custom_lut',
              'custom_cmap',
              'cmapResolution',
+             'gamma',
              'interpolateCmaps',
              'invert',
              'invertClipping',
@@ -234,8 +247,8 @@ def _initPropertyList_MeshOpts(threedee):
     # Remove outline
     # options for 3D
     if threedee:
-        plist.pop(3)
-        plist.pop(2)
+        plist.remove('outlineWidth')
+        plist.remove('outline')
 
     return plist
 
@@ -252,6 +265,14 @@ def _init3DPropertyList_GiftiOpts():
     return []
 
 
+def _initPropertyList_FreesurferOpts(threedee):
+    return []
+
+
+def _init3DPropertyList_FreesurferOpts():
+    return []
+
+
 def _initPropertyList_LabelOpts(threedee):
     return ['lut',
             'outline',
@@ -264,6 +285,7 @@ def _initPropertyList_SHOpts(threedee):
             'shOrder',
             'orientFlip',
             'lighting',
+            'normalise',
             'size',
             'radiusThreshold',
             'colourMode']
@@ -305,6 +327,11 @@ def _initWidgetSpec_ColourMapOpts(threedee):
         'invertClipping'   : props.Widget('invertClipping'),
         'linkLowRanges'    : props.Widget('linkLowRanges'),
         'linkHighRanges'   : props.Widget('linkHighRanges'),
+        'gamma'            : props.Widget(
+            'gamma',
+            showLimits=False,
+            slider=True,
+            spin=True),
         'displayRange'     : props.Widget(
             'displayRange',
             showLimits=False,
@@ -331,13 +358,13 @@ def _initWidgetSpec_VolumeOpts(threedee):
         'volume'         : props.Widget(
             'volume',
             showLimits=False,
-            enabledWhen=lambda o: o.overlay.ndims > 3,
+            enabledWhen=lambda o: o.overlay.ndim > 3,
             spinWidth=6),
         'volumeDim'      : props.Widget(
             'volumeDim',
             showLimits=False,
             slider=False,
-            enabledWhen=lambda o: o.overlay.ndims > 4,
+            enabledWhen=lambda o: o.overlay.ndim > 4,
             spinWidth=2),
         'interpolation'  : props.Widget(
             'interpolation',
@@ -361,17 +388,20 @@ def _initWidgetSpec_VolumeOpts(threedee):
 def _init3DWidgetSpec_VolumeOpts():
 
     return {
-        'dithering'         : props.Widget('dithering',
-                                           showLimits=False),
         'numSteps'          : props.Widget('numSteps',
                                            showLimits=False),
         'blendFactor'       : props.Widget('blendFactor',
                                            showLimits=False),
+        'smoothing'         : props.Widget('smoothing',
+                                           showLimits=False),
         'resolution'        : props.Widget('resolution',
                                            showLimits=False),
         'numClipPlanes'     : props.Widget('numClipPlanes',
                                            slider=False,
                                            showLimits=False),
+        'clipMode'          : props.Widget(
+            'clipMode',
+            labels=strings.choices['Volume3DOpts.clipMode']),
         'showClipPlanes'    : props.Widget('showClipPlanes'),
         'clipPosition'      : props.Widget('clipPosition',
                                            showLimits=False),
@@ -389,17 +419,22 @@ def _initWidgetSpec_MaskOpts(threedee):
         'volume'         : props.Widget(
             'volume',
             showLimits=False,
-            enabledWhen=lambda o: o.overlay.ndims > 3,
+            enabledWhen=lambda o: o.overlay.ndim > 3,
             spinWidth=6),
         'volumeDim'      : props.Widget(
             'volumeDim',
             showLimits=False,
             slider=False,
-            enabledWhen=lambda o: o.overlay.ndims > 4,
+            enabledWhen=lambda o: o.overlay.ndim > 4,
             spinWidth=2),
-        'colour'     : props.Widget('colour'),
-        'invert'     : props.Widget('invert'),
-        'threshold'  : props.Widget('threshold', showLimits=False),
+        'interpolation'  : props.Widget(
+            'interpolation',
+            labels=strings.choices['VolumeOpts.interpolation']),
+        'colour'       : props.Widget('colour'),
+        'invert'       : props.Widget('invert'),
+        'threshold'    : props.Widget('threshold', showLimits=False),
+        'outline'      : props.Widget('outline'),
+        'outlineWidth' : props.Widget('outlineWidth', showLimits=False),
     }
 
 
@@ -412,13 +447,13 @@ def _initWidgetSpec_LabelOpts(threedee):
         'volume'         : props.Widget(
             'volume',
             showLimits=False,
-            enabledWhen=lambda o: o.overlay.ndims > 3,
+            enabledWhen=lambda o: o.overlay.ndim > 3,
             spinWidth=6),
         'volumeDim'      : props.Widget(
             'volumeDim',
             showLimits=False,
             slider=False,
-            enabledWhen=lambda o: o.overlay.ndims > 4,
+            enabledWhen=lambda o: o.overlay.ndim > 4,
             spinWidth=2),
     }
 
@@ -534,6 +569,7 @@ def _initWidgetSpec_SHOpts(threedee):
         'shOrder'    : props.Widget('shOrder'),
         'orientFlip' : props.Widget('orientFlip'),
         'lighting'   : props.Widget('lighting'),
+        'normalise'  : props.Widget('normalise'),
         'size'       : props.Widget(
             'size',
             spin=False,
@@ -590,7 +626,7 @@ def _initWidgetSpec_MeshOpts(threedee):
         if img is None: return 'None'
         else:           return img.name
 
-    def vertexDataName(vdata):
+    def pathName(vdata):
         if vdata is None: return 'None'
         else:             return op.basename(vdata)
 
@@ -619,10 +655,14 @@ def _initWidgetSpec_MeshOpts(threedee):
             labels=strings.choices['MeshOpts.coordSpace'],
             dependencies=['refImage']),
         'colour'       : props.Widget('colour'),
-        'custom_vertexData' : _MeshOpts_VertexDataWidget,
+        'custom_vertexData' : _MeshOpts_vertexDataWidget,
+        'custom_vertexSet'  : _MeshOpts_vertexSetWidget,
+        'vertexSet'    : props.Widget(
+            'vertexSet',
+            labels=pathName),
         'vertexData'   : props.Widget(
             'vertexData',
-            labels=vertexDataName),
+            labels=pathName),
         'vertexDataIndex' : props.Widget(
             'vertexDataIndex',
             showLimits=False,
@@ -707,6 +747,12 @@ def _init3DWidgetSpec_GiftiOpts():
     return {}
 
 
+def _initWidgetSpec_FreesurferOpts(threedee):
+    return {}
+def _init3DWidgetSpec_FreesurferOpts():
+    return {}
+
+
 def _ColourMapOpts_ColourMapWidget(
         target,
         parent,
@@ -735,19 +781,58 @@ def _ColourMapOpts_ColourMapWidget(
     negCmap    = getWidgetSpecs(target, threedee)['negativeCmap']
     useNegCmap = getWidgetSpecs(target, threedee)['useNegativeCmap']
 
+    cbpanel    = imagepanel.ImagePanel(parent)
+    cbpanel.SetMinSize((-1, 30))
+    colourbar  = cbar.ColourBar(overlayList, displayCtx)
+
+    colourbar.bgColour  = (0, 0, 0, 0)
+    colourbar.showLabel = False
+    colourbar.showTicks = False
+
+    def cbarUpdate(*a):
+        w, h = cbpanel.GetSize().Get()
+
+        if w < 20 or h < 20:
+            return
+
+        bmp = colourbar.colourBar(w, h)
+
+        if bmp is None:
+            return
+
+        if fwidgets.wxversion() == fwidgets.WX_PHOENIX:
+            bmp = wx.Bitmap.FromBufferRGBA(w, h, bmp.transpose(1, 0, 2))
+        else:
+            bmp = wx.BitmapFromBufferRGBA( w, h, bmp.transpose(1, 0, 2))
+
+        cbpanel.SetImage(bmp.ConvertToImage())
+
+    lname = 'ColourBarWidget_{}'.format(colourbar)
+
+    def onDestroy(ev):
+        colourbar.deregister(lname)
+        colourbar.destroy()
+
+    colourbar.register(lname,           cbarUpdate)
+    cbpanel.Bind(wx.EVT_SIZE,           cbarUpdate)
+    cbpanel.Bind(wx.EVT_WINDOW_DESTROY, onDestroy)
+
+    cbarUpdate()
+
     cmap       = props.buildGUI(parent, target, cmap)
     negCmap    = props.buildGUI(parent, target, negCmap)
     useNegCmap = props.buildGUI(parent, target, useNegCmap)
 
     useNegCmap.SetLabel(strings.properties[target, 'useNegativeCmap'])
 
-    sizer = wx.FlexGridSizer(2, 2, 0, 0)
+    sizer = wx.GridBagSizer()
     sizer.AddGrowableCol(0)
 
-    sizer.Add(cmap,       flag=wx.EXPAND)
-    sizer.Add(loadButton, flag=wx.EXPAND)
-    sizer.Add(negCmap,    flag=wx.EXPAND)
-    sizer.Add(useNegCmap, flag=wx.EXPAND)
+    sizer.Add(cbpanel,    (0, 0), (1, 2), flag=wx.EXPAND)
+    sizer.Add(cmap,       (1, 0), (1, 1), flag=wx.EXPAND)
+    sizer.Add(loadButton, (1, 1), (1, 1), flag=wx.EXPAND)
+    sizer.Add(negCmap,    (2, 0), (1, 1), flag=wx.EXPAND)
+    sizer.Add(useNegCmap, (2, 1), (1, 1), flag=wx.EXPAND)
 
     return sizer, [cmap, negCmap, useNegCmap]
 
@@ -831,9 +916,9 @@ def _VolumeOpts_3DClipPlanes(
     #
     # TODO what is the lifespan of this listener?
     def numClipPlanesChanged(*a):
-        if fslplatform.isWidgetAlive(panel) and \
-           fslplatform.isWidgetAlive(parent):
-            async.idle(panel.updateWidgets, target, '3d')
+        if fwidgets.isalive(panel) and \
+           fwidgets.isalive(parent):
+            idle.idle(panel.updateWidgets, target, '3d')
 
     name = '{}_{}_VolumeOpts_3DClipPlanes'.format(
         target.name, id(panel))
@@ -844,23 +929,23 @@ def _VolumeOpts_3DClipPlanes(
                        overwrite=True,
                        weak=False)
 
+    numPlanes    = target.numClipPlanes
     numPlaneSpec = get3DWidgetSpecs(target)['numClipPlanes']
+    clipMode     = get3DWidgetSpecs(target)['clipMode']
     showPlanes   = get3DWidgetSpecs(target)['showClipPlanes']
     position     = get3DWidgetSpecs(target)['clipPosition']
     azimuth      = get3DWidgetSpecs(target)['clipAzimuth']
     inclination  = get3DWidgetSpecs(target)['clipInclination']
 
-    numPlanes = target.numClipPlanes
+    specs = [numPlaneSpec, showPlanes, clipMode]
 
     if numPlanes == 0:
-        return [numPlaneSpec, showPlanes], None
+        return specs, None
 
     positions    = [copy.deepcopy(position)    for i in range(numPlanes)]
     azimuths     = [copy.deepcopy(azimuth)     for i in range(numPlanes)]
     inclinations = [copy.deepcopy(inclination) for i in range(numPlanes)]
 
-    specs = [numPlaneSpec, showPlanes]
-
     for i in range(numPlanes):
 
         positions[i]   .index = i
@@ -875,7 +960,7 @@ def _VolumeOpts_3DClipPlanes(
     return specs, None
 
 
-def _MeshOpts_VertexDataWidget(
+def _MeshOpts_vertexDataWidget(
         target,
         parent,
         panel,
@@ -905,6 +990,38 @@ def _MeshOpts_VertexDataWidget(
     return sizer, [vdata]
 
 
+def _MeshOpts_vertexSetWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains a widget for controlling the
+    :attr:`.MeshOpts.vertexSet` property, and also has a button
+    which opens a file dialog, allowing the user to select other
+    data.
+    """
+
+    loadAction = loadvdata.LoadVertexDataAction(overlayList,
+                                                displayCtx,
+                                                vertices=True)
+    loadButton = wx.Button(parent)
+    loadButton.SetLabel(strings.labels[panel, 'loadVertices'])
+
+    loadAction.bindToWidget(panel, wx.EVT_BUTTON, loadButton)
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+    vdata = getWidgetSpecs(target, threedee)['vertexSet']
+    vdata = props.buildGUI(parent, target, vdata)
+
+    sizer.Add(vdata,      flag=wx.EXPAND, proportion=1)
+    sizer.Add(loadButton, flag=wx.EXPAND)
+
+    return sizer, [vdata]
+
+
 def _MeshOpts_LutWidget(
         target,
         parent,
