@@ -109,6 +109,23 @@ class ColourMapOpts(object):
     """The colour map, a :class:`matplotlib.colors.Colourmap` instance."""
 
 
+    gamma = props.Real(minval=-1, maxval=1, clamped=True, default=0)
+    """Gamma correction factor - exponentially weights the :attr:`cmap`
+    and :attr:`negCmap` towards the low or high ends.
+
+    This property takes values between -1 and +1. The exponential weight
+    that should actually be used to apply gamma correction should be derived
+    as follows:
+
+      - -1 corresponds to a gamma of 0.01
+      -  0 corresponds to a gamma of 1
+      - +1 corresponds to a gamma of 10
+
+    The :meth:`realGamma` method will apply this scaling and return the
+    exponent to be used.
+    """
+
+
     cmapResolution = props.Int(minval=2, maxval=1024, default=256)
     """Resolution for the colour map, i.e. the number of colours to use. """
 
@@ -159,6 +176,23 @@ class ColourMapOpts(object):
     """
 
 
+    @staticmethod
+    def realGamma(gamma):
+        """Return the value of ``gamma`` property, scaled appropriately.
+        for use as an exponent.
+        """
+
+        # a gamma in the range [-1, 0]
+        # gets scaled to [0.01, 1]
+        if gamma < 0:
+            return (gamma + 1.01) * 0.99
+
+        # a gamma in the range [0, 1]
+        # gets scaled to [1, 10]
+        else:
+            return 1 + 9 * gamma
+
+
     def __init__(self):
         """Create a ``ColourMapOpts`` instance. This must be called
         *after* the :meth:`.DisplayOpts.__init__` method.
@@ -238,8 +272,7 @@ class ColourMapOpts(object):
         # to the parent.
         if (not self.__registered) or \
            (not self.isSyncedToParent('displayRange')):
-            self.updateDataRange()
-
+            self.updateDataRange(False, False)
 
 
     def getColourMapOptsListenerName(self):
@@ -332,10 +365,15 @@ class ColourMapOpts(object):
                       property will be reset to the data range returned
                       by :meth:`getDataRange`. Otherwise the existing
                       value will be preserved.
+
         :arg resetCR: If ``True`` (the default), the :attr:`clippingRange`
                       property will be reset to the clipping range returned
                       by :meth:`getClippingRange`. Otherwise the existing
                       value will be preserved.
+
+        Note that both of these flags will be ignored if the existing low/high
+        :attr:`displayRange`/:attr:`clippingRange` values and limits are equal
+        to each other.
         """
 
         dataMin, dataMax = self.getDataRange()
@@ -372,11 +410,13 @@ class ColourMapOpts(object):
             # If display/clipping limit range
             # is 0, we assume that they haven't
             # yet been set
-            drUnset = self.displayRange .xmin == self.displayRange .xmax
-            crUnset = self.clippingRange.xmin == self.clippingRange.xmax
-            crGrow  = self.clippingRange.xhi  == self.clippingRange.xmax
-            drUnset = resetDR or drUnset
-            crUnset = resetCR or crUnset
+            drUnset = (self.displayRange .xmin == self.displayRange .xmax and
+                       self.displayRange .xlo  == self.displayRange .xhi)
+            crUnset = (self.clippingRange.xmin == self.clippingRange.xmax and
+                       self.clippingRange.xlo  == self.clippingRange.xhi)
+            crGrow  =  self.clippingRange.xhi  == self.clippingRange.xmax
+            drUnset =  resetDR or drUnset
+            crUnset =  resetCR or crUnset
 
             log.debug('[{}] Updating range limits [dr: {} - {}, ''cr: '
                       '{} - {}]'.format(id(self), drmin, drmax, crmin, crmax))
@@ -518,7 +558,7 @@ class ColourMapOpts(object):
             self.display.enableProperty('contrast')
 
         if kwa.pop('updateDataRange', True):
-            self.updateDataRange()
+            self.updateDataRange(resetDR=False, resetCR=False)
 
 
     def __linkLowRangesChanged(self, *a):
