@@ -225,7 +225,7 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         if self.__child:
 
             # is this a >3D volume?
-            ndims = self.overlay.ndims
+            ndims = self.overlay.ndim
 
             # We store indices for every dimension
             # past the XYZ dims. Whenever the volumeDim
@@ -304,7 +304,7 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         value of ``volume`` for the new ``volumeDim``.
         """
 
-        if self.overlay.ndims <= 3:
+        if self.overlay.ndim <= 3:
             return
 
         # Here we disable volumeDim listeners on all
@@ -546,6 +546,14 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         self.__xforms['texture', 'reference']   = texToRefMat
 
 
+    @classmethod
+    def getVolumeProps(cls):
+        """Overrides :meth:`DisplayOpts.getVolumeProps`. Returns a list
+        of property names which control the displayed volume/timepoint.
+        """
+        return ['volume', 'volumeDim']
+
+
     def getTransform(self, from_, to, xform=None):
         """Return a matrix which may be used to transform coordinates
         from ``from_`` to ``to``. Valid values for ``from_`` and ``to``
@@ -729,7 +737,14 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         return voxels
 
 
-    def transformCoords(self, coords, from_, to_, vround=False):
+    def transformCoords(self,
+                        coords,
+                        from_,
+                        to_,
+                        vround=False,
+                        vector=False,
+                        pre=None,
+                        post=None):
         """Transforms the given coordinates from ``from_`` to ``to_``.
 
         The ``from_`` and ``to_`` parameters must be those accepted by the
@@ -741,11 +756,21 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         :arg vround: If ``True``, and ``to_ in ('voxel', 'id)``, the
                      transformed coordinates are rounded to the nearest
                      integer.
+        :arg vector: Defaults to ``False``. If ``True``, the coordinates
+                     are treated as vectors.
+        :arg pre:    Transformation to apply before the ``from_``-to-``to``
+                     transformation.
+        :arg post:   Transformation to apply after the ``from_``-to-``to``
+                     transformation.
         """
 
-        xform  = self.getTransform(from_, to_)
+        xform = self.getTransform(from_, to_)
+
+        if pre  is not None: xform = transform.concat(xform, pre)
+        if post is not None: xform = transform.concat(post, xform)
+
         coords = np.array(coords)
-        coords = transform.transform(coords, xform)
+        coords = transform.transform(coords, xform, vector=vector)
 
         # Round to integer voxel coordinates?
         if to_ in ('voxel', 'id') and vround:
@@ -816,10 +841,10 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         if slc is None:
             slc = [slice(None), slice(None), slice(None)]
 
-        if self.overlay.ndims <= 3:
+        if self.overlay.ndim <= 3:
             return tuple(slc)
 
-        newSlc      = [None] * self.overlay.ndims
+        newSlc      = [None] * self.overlay.ndim
         newSlc[:3]  = slc
         newSlc[ 3:] = self.getAttribute('volume', 'cache')
 
@@ -855,6 +880,46 @@ class VolumeOpts(cmapopts.ColourMapOpts, vol3dopts.Volume3DOpts, NiftiOpts):
     """
 
 
+    @classmethod
+    def getInitialDisplayRange(cls):
+        """This class method returns a tuple containing ``(low, high)``
+        percentile values which are used to set the initial values for the
+        :attr:`.ColourMapOpts.displayRange` and
+        :attr:`.ColourMapOpts.clippingRange` properties. If the initial
+        display range has not yet been set (via the
+        :meth:`setInitialDisplayRange` method), ``None`` is returned.
+        """
+        try:
+            return cls.__initialDisplayRange
+        except AttributeError:
+            return None
+
+
+    @classmethod
+    def setInitialDisplayRange(cls, drange):
+        """Sets the initial values for the :attr:`.ColourMapOpts.displayRange`
+        and :attr:`.ColourMapOpts.clippingRange` to be used for new
+        :class:`VolumeOpts` instances.
+
+        :arg drange: A tuple containing ``(low, high)`` display range values
+                     as percentiles of the image data range. May be ``None``,
+                     in which case the initial display range will be set to the
+                     image data range.
+        """
+
+        if drange is not None:
+            low, high = drange
+            if not all((low < high,
+                        low >= 0,
+                        low <= 100,
+                        high >= 0,
+                        high <= 100)):
+                raise ValueError('Invalid initial display '
+                                 'range: {}'.format(drange))
+
+        cls.__initialDisplayRange = drange
+
+
     def __init__(self,
                  overlay,
                  display,
@@ -906,6 +971,28 @@ class VolumeOpts(cmapopts.ColourMapOpts, vol3dopts.Volume3DOpts, NiftiOpts):
                            overlayList,
                            displayCtx,
                            **kwargs)
+
+        # Some things only happen
+        # on the parent instance
+        self.__registered = self.getParent() is None
+
+        # Configure the initial display
+        # range for new images, from the
+        # initialDisplayRange percentiles.
+        # We do this before ColourMapOpts.init
+        drange = VolumeOpts.getInitialDisplayRange()
+
+        if self.__registered and drange is not None:
+
+            if   overlay.ndim == 3: sample = overlay[:]
+            elif overlay.ndim == 4: sample = overlay[..., 0]
+
+            drange = np.percentile(sample[sample != 0], drange)
+            crange = [drange[0], overlay.dataRange[1]]
+
+            self.displayRange  = drange
+            self.clippingRange = crange
+
         cmapopts .ColourMapOpts.__init__(self)
         vol3dopts.Volume3DOpts .__init__(self)
 
@@ -943,7 +1030,6 @@ class VolumeOpts(cmapopts.ColourMapOpts, vol3dopts.Volume3DOpts, NiftiOpts):
         # will change the display data range. These
         # cannot be unbound between parent/children,
         # so only the parent needs to listen.
-        self.__registered = self.getParent() is None
         if self.__registered:
             overlayList.addListener('overlays',
                                     self.name,
