@@ -13,13 +13,13 @@ from __future__ import division
 
 import logging
 import collections
-
+import deprecation
 import six
 
 import wx
 import wx.lib.agw.aui                     as aui
 
-import fsl.utils.async                    as async
+import fsl.utils.idle                     as idle
 import fsl.utils.settings                 as fslsettings
 from   fsl.utils.platform import platform as fslplatform
 import fsleyes_widgets.dialog             as fsldlg
@@ -31,7 +31,7 @@ import fsleyes.profiles.shortcuts         as shortcuts
 
 from . import actions
 from . import tooltips
-from . import perspectives
+from . import layouts
 from . import displaycontext
 
 
@@ -72,31 +72,31 @@ class FSLeyesFrame(wx.Frame):
     they can be restored the next time a ``FSLeyesFrame`` is opened. The
     settings are saved using the :class:`~fsl.utils.settings` module.
     Currently, the frame position, size, and layout (see the
-    :mod:`.perspectives` module) are saved.
+    :mod:`.layout` module) are saved.
 
 
     **Programming interface**
 
 
-    The ``FSLeyesFrame`` provides the following methods for programmatically
-    configuring the display:
+    The ``FSLeyesFrame`` provides the following properties and methods for
+    programmatically configuring the display:
 
     .. autosummary::
        :nosignatures:
 
-       getOverlayList
-       getDisplayContext
-       getViewPanels
+       overlayList
+       displayCtx
+       viewPanels
+       focusedViewPanel
+       auiManager
        getViewPanelInfo
        getViewPanelID
        getViewPanelTitle
-       getFocusedViewPanel
        addViewPanel
        viewPanelDefaultLayout
        removeViewPanel
        removeAllViewPanels
-       getAuiManager
-       refreshPerspectiveMenu
+       refreshLayoutMenu
        runScript
        populateMenu
        Close
@@ -124,7 +124,8 @@ class FSLeyesFrame(wx.Frame):
                  overlayList,
                  displayCtx,
                  restore=False,
-                 save=True):
+                 save=True,
+                 fontSize=None):
         """Create a ``FSLeyesFrame``.
 
         :arg parent:      The :mod:`wx` parent object.
@@ -137,6 +138,8 @@ class FSLeyesFrame(wx.Frame):
                           view panels will be displayed.
 
         :arg save:        Save current layout when closed.
+
+        :arg fontSize:    Application-wide font size to use. Defaults to 10.
         """
 
         wx.Frame.__init__(self, parent, title='FSLeyes')
@@ -147,9 +150,10 @@ class FSLeyesFrame(wx.Frame):
         # inherited by all child controls.
         font = self.GetFont()
 
-        if fslplatform.wxPlatform == fslplatform.WX_GTK: font.SetPointSize(8)
-        else:                                            font.SetPointSize(10)
+        if fontSize is None:
+            fontSize = 10
 
+        font.SetPointSize(fontSize)
         font.SetWeight(wx.FONTWEIGHT_LIGHT)
         self.SetFont(font)
 
@@ -222,7 +226,7 @@ class FSLeyesFrame(wx.Frame):
                     self.__statusBar.SetLabel(msg)
                     self.__statusBar.Refresh()
                     self.__statusBar.Update()
-                except:
+                except Exception:
                     pass
 
             wx.CallAfter(realUpdate)
@@ -251,7 +255,7 @@ class FSLeyesFrame(wx.Frame):
 
         # Refs to menus
         self.__menuBar        = None
-        self.__perspMenu      = None
+        self.__layoutMenu     = None
         self.__recentPathMenu = None
 
         # The recent paths manager notifies us when
@@ -259,7 +263,7 @@ class FSLeyesFrame(wx.Frame):
         # __makeRecentPathsMenu methods
         import fsleyes.actions.loadoverlay as loadoverlay
         loadoverlay.recentPathManager.register(
-            'FSLeyesFrame', self.__makeRecentPathsMenu)
+            self.__name, self.__makeRecentPathsMenu)
 
         # This dictionary contains mappings of the form
         #
@@ -290,6 +294,51 @@ class FSLeyesFrame(wx.Frame):
         self.Layout()
 
 
+    @property
+    def overlayList(self):
+        """Returns the :class:`.OverlayList` which contains the overlays
+        being displayed by this ``FSLeyesFrame``.
+        """
+        return self.__overlayList
+
+
+    @property
+    def displayCtx(self):
+        """Returns the top-level :class:`.DisplayContext` associated with this
+        ``FSLeyesFrame``.
+        """
+        return self.__displayCtx
+
+
+    @property
+    def viewPanels(self):
+        """Returns a list of all :class:`.ViewPanel` instances that are
+        currenlty displayed in this ``FSLeyesFrame``.
+        """
+        return list(self.__viewPanels)
+
+
+    @property
+    def focusedViewPanel(self):
+        """Returns the :class:`.ViewPanel` which currently has focus, or
+        ``None`` if no ``ViewPanel`` has focus.
+        """
+        import fsleyes.views.viewpanel as viewpanel
+
+        focused = wx.Window.FindFocus()
+
+        while focused is not None:
+
+            if isinstance(focused, viewpanel.ViewPanel):
+                return focused
+
+            focused = focused.GetParent()
+        return None
+
+
+    @deprecation.deprecated(deprecated_in='0.16.0',
+                            removed_in='1.0.0',
+                            details='Use overlayList instead')
     def getOverlayList(self):
         """Returns the :class:`.OverlayList` which contains the overlays
         being displayed by this ``FSLeyesFrame``.
@@ -297,6 +346,9 @@ class FSLeyesFrame(wx.Frame):
         return self.__overlayList
 
 
+    @deprecation.deprecated(deprecated_in='0.16.0',
+                            removed_in='1.0.0',
+                            details='Use displayCtx instead')
     def getDisplayContext(self):
         """Returns the top-level :class:`.DisplayContext` associated with this
         ``FSLeyesFrame``.
@@ -304,6 +356,9 @@ class FSLeyesFrame(wx.Frame):
         return self.__displayCtx
 
 
+    @deprecation.deprecated(deprecated_in='0.16.0',
+                            removed_in='1.0.0',
+                            details='Use viewPanels instead')
     def getViewPanels(self):
         """Returns a list of all :class:`.ViewPanel` instances that are
         currenlty displayed in this ``FSLeyesFrame``.
@@ -311,6 +366,16 @@ class FSLeyesFrame(wx.Frame):
         return list(self.__viewPanels)
 
 
+    @deprecation.deprecated(deprecated_in='0.16.0',
+                            removed_in='1.0.0',
+                            details='Use focusedViewPanel instead')
+    def getFocusedViewPanel(self):
+        """Returns the :class:`.ViewPanel` which currently has focus, or
+        ``None`` if no ``ViewPanel`` has focus.
+        """
+        return self.focusedViewPanel
+
+
     def getViewPanelID(self, viewPanel):
         """Returns the ID that was assigned to the given :class:`.ViewPanel`.
         This is a sequentially increasing integer, starting from 1.
@@ -332,6 +397,17 @@ class FSLeyesFrame(wx.Frame):
         return self.__auiManager.GetPane(viewPanel)
 
 
+    @property
+    def auiManager(self):
+        """Returns the ``wx.lib.agw.aui.AuiManager` object which is managing
+        the layout of this ``FSLeyesFrame``.
+        """
+        return self.__auiManager
+
+
+    @deprecation.deprecated(deprecated_in='0.16.0',
+                            removed_in='1.0.0',
+                            details='Use auiManager instead')
     def getAuiManager(self):
         """Returns the ``wx.lib.agw.aui.AuiManager` object which is managing
         the layout of this ``FSLeyesFrame``.
@@ -351,23 +427,6 @@ class FSLeyesFrame(wx.Frame):
         self.__auiManager.Update()
 
 
-    def getFocusedViewPanel(self):
-        """Returns the :class:`.ViewPanel` which currently has focus, or
-        ``None`` if no ``ViewPanel`` has focus.
-        """
-        import fsleyes.views.viewpanel as viewpanel
-
-        focused = wx.Window.FindFocus()
-
-        while focused is not None:
-
-            if isinstance(focused, viewpanel.ViewPanel):
-                return focused
-
-            focused = focused.GetParent()
-        return None
-
-
     def removeAllViewPanels(self):
         """Removes all view panels from this ``FSLeyesFrame``.
 
@@ -401,6 +460,8 @@ class FSLeyesFrame(wx.Frame):
 
         :arg panelCls: The :class:`.ViewPanel` type to be added.
 
+        :returns: The newly created ``ViewPanel``.
+
         All other arguments are passed to the ``__init__`` method of the child
         :class:`.DisplayContext` that is created for the new view.
         """
@@ -414,7 +475,7 @@ class FSLeyesFrame(wx.Frame):
 
         # The PaneInfo Name contains the panel
         # class name - this is used for saving
-        # and restoring perspectives .
+        # and restoring layouts .
         name  = '{} {}'.format(panelCls.__name__,        panelId)
         title = '{} {}'.format(strings.titles[panelCls], panelId)
 
@@ -532,12 +593,23 @@ class FSLeyesFrame(wx.Frame):
             viewPanel.toggleLocationPanel()
 
         elif isinstance(viewPanel, Scene3DPanel):
-            pass
+            viewPanel.toggleDisplayToolBar()
+            viewPanel.toggleScene3DToolBar()
+            viewPanel.toggleOverlayList()
+            viewPanel.toggleLocationPanel()
 
 
+    def refreshLayoutMenu(self):
+        """Re-creates the *View -> Layouts* sub-menu. """
+        self.__makeLayoutMenu()
+
+
+    @deprecation.deprecated(deprecated_in='0.24.0',
+                            removed_in='1.0.0',
+                            details='Use refreshLayoutMenu')
     def refreshPerspectiveMenu(self):
-        """Re-creates the *View -> Perspectives* sub-menu. """
-        self.__makePerspectiveMenu()
+        """Deprecated."""
+        self.refreshLayoutMenu()
 
 
     def runScript(self, script=None):
@@ -721,7 +793,7 @@ class FSLeyesFrame(wx.Frame):
                           :class:`.ViewPanel`.
 
         :arg runOnIdle:   If ``True``, the action is executed on the
-                          ``async.idle`` loop. Otherwise (the default),
+                          ``idle.idle`` loop. Otherwise (the default),
                           the action is executed on the calling thread.
         """
 
@@ -741,7 +813,7 @@ class FSLeyesFrame(wx.Frame):
         if ignoreFocus or shortcut is None or (not keyDown):
             func = target.getAction(actionName)
             if runOnIdle:
-                async.idle(func)
+                idle.idle(func)
             else:
                 func()
             return
@@ -769,7 +841,7 @@ class FSLeyesFrame(wx.Frame):
         # ViewPanel/action to execute based on the
         # keyboard shortcut.
 
-        viewPanel  = self.getFocusedViewPanel()
+        viewPanel  = self.focusedViewPanel
         actionName = self.__viewPanelShortcuts[shortcut].get(viewPanel, None)
 
         if actionName is None:
@@ -778,7 +850,7 @@ class FSLeyesFrame(wx.Frame):
         func = viewPanel.getAction(actionName)
 
         if runOnIdle:
-            async.idle(func)
+            idle.idle(func)
         else:
             func()
 
@@ -886,9 +958,9 @@ class FSLeyesFrame(wx.Frame):
     def __configDisplaySync(self, newPanel=None):
         """Called by :meth:`addViewPanel` and :meth:`__onViewPanelClose`.
 
-        This method ensures that at the display properties and overlay order
-        for the :class:`.DisplayContext` of at least least one
-        :class:`.CanvasPanel` is synced to the master
+        This method ensures that the display properties, overlay order,
+        and selected overlay for the :class:`.DisplayContext` of at least
+        least one :class:`.CanvasPanel` is synced to the master
         :class:`.DisplayContext`.
 
         :arg newPanel: If this method has been called as a result of a new
@@ -897,14 +969,22 @@ class FSLeyesFrame(wx.Frame):
         """
 
         import fsleyes.views.canvaspanel as canvaspanel
+        import fsleyes.views.plotpanel   as plotpanel
+
+        # Plot panel defaults
+        if newPanel is not None and isinstance(newPanel, plotpanel.PlotPanel):
+            newPanel.displayCtx.syncOverlayDisplay = True
+            newPanel.displayCtx.syncOverlayVolume  = True
+            newPanel.displayCtx.unsyncFromParent('overlayOrder')
+            newPanel.displayCtx.unsyncFromParent('selectedOverlay')
 
         canvasPanels = [vp for vp in self.__viewPanels
                         if isinstance(vp, canvaspanel.CanvasPanel)]
-        canvasCtxs   = [c.getDisplayContext() for c in canvasPanels]
+        canvasCtxs   = [c.displayCtx for c in canvasPanels]
         numCanvases  = len(canvasPanels)
 
-        # We only care about
-        # canvas panels.
+        # We only care about canvas
+        # panels from here on.
         if numCanvases == 0:
             return
 
@@ -915,20 +995,28 @@ class FSLeyesFrame(wx.Frame):
         displaySynced = any([c.syncOverlayDisplay
                              for c in canvasCtxs
                              if c is not newPanel])
+        volumeSynced = any([c.syncOverlayVolume
+                             for c in canvasCtxs
+                             if c is not newPanel])
         orderSynced   = any([c.isSyncedToParent('overlayOrder')
                              for c in canvasCtxs
                              if c is not newPanel])
+        selOvlSynced  = any([c.isSyncedToParent('selectedOverlay')
+                             for c in canvasCtxs
+                             if c is not newPanel])
 
         # If an existing canvas panel is
         # already synced to the master,
         # then we set the new panel to
         # be unsynced
-        if displaySynced and orderSynced:
+        if displaySynced and volumeSynced and orderSynced and selOvlSynced:
             if newPanel is not None and \
                isinstance(newPanel, canvaspanel.CanvasPanel):
-                childDC = newPanel.getDisplayContext()
+                childDC                    = newPanel.displayCtx
+                childDC.syncOverlayVolume  = True
                 childDC.syncOverlayDisplay = False
                 childDC.unsyncFromParent('overlayOrder')
+                childDC.unsyncFromParent('selectedOverlay')
 
         # If no existing CanvasPanels are
         # synced to the master context,
@@ -937,7 +1025,7 @@ class FSLeyesFrame(wx.Frame):
         else:
             if newPanel is not None: panel = newPanel
             else:                    panel = canvasPanels[-1]
-            childDC = panel.getDisplayContext()
+            childDC = panel.displayCtx
 
             # Make sure that the parent context
             # inherits the values from this context
@@ -946,18 +1034,20 @@ class FSLeyesFrame(wx.Frame):
             childDC.setBindingDirection(False)
 
             for display in displays:
-                opts = display.getDisplayOpts()
+                opts = display.opts
                 display.setBindingDirection(False)
                 opts   .setBindingDirection(False)
 
             childDC.syncOverlayDisplay = True
+            childDC.syncOverlayVolume  = True
             childDC.syncToParent('overlayOrder')
+            childDC.syncToParent('selectedOverlay')
 
             # Reset the binding directiona
             childDC.setBindingDirection(True)
 
             for display in displays:
-                opts = display.getDisplayOpts()
+                opts = display.opts
                 display.setBindingDirection(True)
                 opts   .setBindingDirection(True)
 
@@ -1056,7 +1146,7 @@ class FSLeyesFrame(wx.Frame):
                                   not dlg.CheckBoxState())
 
             if save:
-                layout   = perspectives.serialisePerspective(self)
+                layout   = layouts.serialiseLayout(self)
                 size     = self.GetSize().Get()
                 position = self.GetScreenPosition().Get()
 
@@ -1078,6 +1168,11 @@ class FSLeyesFrame(wx.Frame):
         for panel in self.__viewPanels:
             panel.destroy()
 
+        # Deregister loadoverlay listener
+        # (registered in __init__)
+        import fsleyes.actions.loadoverlay as loadoverlay
+        loadoverlay.recentPathManager.deregister(self.__name)
+
 
     def __restoreState(self, restore):
         """Called by :meth:`__init__`.
@@ -1179,19 +1274,19 @@ class FSLeyesFrame(wx.Frame):
                 log.debug('Restoring previous layout: {}'.format(layout))
 
                 try:
-                    perspectives.applyPerspective(
+                    layouts.applyLayout(
                         self,
                         'fsleyes.frame.layout',
                         layout,
                         message=strings.messages[self, 'restoringLayout'])
-                except:
+                except Exception:
                     log.warn('Previous layout could not be restored - '
                              'falling back to default layout.')
                     log.debug('Layout restore error', exc_info=True)
                     layout = None
 
             if layout is None:
-                perspectives.loadPerspective(self, 'default')
+                layouts.loadLayout(self, 'default')
 
 
     def __makeMenuBar(self):
@@ -1224,15 +1319,15 @@ class FSLeyesFrame(wx.Frame):
         else:
             fsleyesMenu = None
 
-        fileMenu        = wx.Menu()
-        overlayMenu     = wx.Menu()
-        viewMenu        = wx.Menu()
-        perspectiveMenu = wx.Menu()
-        settingsMenu    = wx.Menu()
-        toolsMenu       = wx.Menu()
+        fileMenu     = wx.Menu()
+        overlayMenu  = wx.Menu()
+        viewMenu     = wx.Menu()
+        layoutMenu   = wx.Menu()
+        settingsMenu = wx.Menu()
+        toolsMenu    = wx.Menu()
 
-        self.__menuBar   = menuBar
-        self.__perspMenu = perspectiveMenu
+        self.__menuBar    = menuBar
+        self.__layoutMenu = layoutMenu
 
         menuBar.Append(fileMenu,     'File')
         menuBar.Append(overlayMenu,  'Overlay')
@@ -1266,10 +1361,10 @@ class FSLeyesFrame(wx.Frame):
 
         self.__makeViewPanelMenu(viewMenu)
 
-        # Perspectives
+        # Layouts
         viewMenu.AppendSeparator()
-        viewMenu.AppendSubMenu(perspectiveMenu, 'Perspectives')
-        self.__makePerspectiveMenu()
+        viewMenu.AppendSubMenu(layoutMenu, 'Layouts')
+        self.__makeLayoutMenu()
 
 
     def __makeFSLeyesMenu(self, menu):
@@ -1325,14 +1420,20 @@ class FSLeyesFrame(wx.Frame):
 
         from fsleyes.actions.loadoverlay        import LoadOverlayAction
         from fsleyes.actions.loadoverlayfromdir import LoadOverlayFromDirAction
+        from fsleyes.actions.loaddicom          import LoadDicomAction
         from fsleyes.actions.loadstandard       import LoadStandardAction
         from fsleyes.actions.loadatlas          import LoadAtlasAction
+        from fsleyes.actions.browsexnat         import BrowseXNATAction
         from fsleyes.actions.runscript          import RunScriptAction
+        from fsleyes.actions.notebook           import NotebookAction
 
         fileActions = [LoadOverlayAction,
                        LoadOverlayFromDirAction,
                        LoadStandardAction,
+                       LoadDicomAction,
+                       BrowseXNATAction,
                        LoadAtlasAction,
+                       NotebookAction,
                        RunScriptAction]
 
         for action in fileActions:
@@ -1444,67 +1545,67 @@ class FSLeyesFrame(wx.Frame):
             self.Bind(wx.EVT_MENU, action, item)
 
 
-    def __makePerspectiveMenu(self):
-        """Called by :meth:`__makeMenuBar` and :meth:`refreshPerspectiveMenu`.
-        Re-creates the *View->Perspectives* menu.
+    def __makeLayoutMenu(self):
+        """Called by :meth:`__makeMenuBar` and :meth:`refreshLayoutMenu`.
+        Re-creates the *View->Layouts* menu.
         """
 
-        from fsleyes.actions.loadperspective  import LoadPerspectiveAction
-        from fsleyes.actions.saveperspective  import SavePerspectiveAction
-        from fsleyes.actions.clearperspective import ClearPerspectiveAction
+        from fsleyes.actions.loadlayout   import LoadLayoutAction
+        from fsleyes.actions.savelayout   import SaveLayoutAction
+        from fsleyes.actions.clearlayouts import ClearLayoutsAction
 
-        perspMenu = self.__perspMenu
+        layoutMenu = self.__layoutMenu
 
         # Remove any existing menu items
-        for item in perspMenu.GetMenuItems():
-            perspMenu.Delete(item.GetId())
+        for item in layoutMenu.GetMenuItems():
+            layoutMenu.Delete(item.GetId())
 
-        builtIns = list(perspectives.BUILT_IN_PERSPECTIVES.keys())
-        saved    = perspectives.getAllPerspectives()
+        builtIns = list(layouts.BUILT_IN_LAYOUTS.keys())
+        saved    = layouts.getAllLayouts()
 
-        # Add a menu item to load each built-in perspectives
-        for persp in builtIns:
+        # Add a menu item to load each built-in layouts
+        for layout in builtIns:
 
-            title    = strings.perspectives.get(persp, persp)
-            shortcut = shortcuts.actions.get((self, 'perspectives', persp),
+            title    = strings.layouts.get(layout, layout)
+            shortcut = shortcuts.actions.get((self, 'layouts', layout),
                                              None)
 
             if shortcut is not None:
                 title = '{}\t{}'.format(title, shortcut)
 
-            menuItem = perspMenu.Append(wx.ID_ANY, title)
+            menuItem = layoutMenu.Append(wx.ID_ANY, title)
 
-            actionObj = LoadPerspectiveAction(self, persp)
+            actionObj = LoadLayoutAction(self, layout)
             actionObj.bindToWidget(self, wx.EVT_MENU, menuItem)
 
         if len(builtIns) > 0:
-            perspMenu.AppendSeparator()
+            layoutMenu.AppendSeparator()
 
-        # Add a menu item to load each saved perspective
-        for persp in saved:
+        # Add a menu item to load each saved layout
+        for layout in saved:
 
-            menuItem  = perspMenu.Append(
-                wx.ID_ANY, strings.perspectives.get(persp, persp))
-            actionObj = LoadPerspectiveAction(self, persp)
+            menuItem  = layoutMenu.Append(
+                wx.ID_ANY, strings.layouts.get(layout, layout))
+            actionObj = LoadLayoutAction(self, layout)
             actionObj.bindToWidget(self, wx.EVT_MENU, menuItem)
 
-        # Add menu items for other perspective
-        # operations, but separate them from the
-        # existing perspectives
+        # Add menu items for other layout
+        # operations, but separate them
+        # from the existing layouts
         if len(saved) > 0:
-            perspMenu.AppendSeparator()
+            layoutMenu.AppendSeparator()
 
-        # TODO: Delete a single perspective?
+        # TODO: Delete a single layout?
         #       Save to/load from file?
-        perspActions = [SavePerspectiveAction,
-                        ClearPerspectiveAction]
+        layoutActions = [SaveLayoutAction,
+                         ClearLayoutsAction]
 
-        for pa in perspActions:
+        for la in layoutActions:
 
-            actionObj     = pa(self)
-            perspMenuItem = perspMenu.Append(wx.ID_ANY, strings.actions[pa])
+            actionObj      = la(self)
+            layoutMenuItem = layoutMenu.Append(wx.ID_ANY, strings.actions[la])
 
-            actionObj.bindToWidget(self, wx.EVT_MENU, perspMenuItem)
+            actionObj.bindToWidget(self, wx.EVT_MENU, layoutMenuItem)
 
 
     def __makeOverlayMenu(self, menu):
@@ -1618,7 +1719,7 @@ class FSLeyesFrame(wx.Frame):
         # ensure that the tools for different view
         # panel types are always in a consistent
         # order (i.e. ortho first, shell last)
-        panels  = self.getViewPanels()
+        panels  = self.viewPanels
         vpOrder = [OrthoPanel,
                    LightBoxPanel,
                    Scene3DPanel,
