@@ -124,7 +124,6 @@ be displayed as:
    ~fsleyes.gl.gllinevector.GLLineVector
    ~fsleyes.gl.glrgbvector.GLRGBVector
    ~fsleyes.gl.glmesh.GLMesh
-   ~fsleyes.gl.glgiftimesh.GLGiftiMesh
    ~fsleyes.gl.gltensor.GLTensor
    ~fsleyes.gl.glsh.GLSH
 
@@ -179,9 +178,9 @@ import os
 import logging
 import platform
 
-import fsl.utils.async                    as async
+import fsl.utils.idle                     as idle
 from   fsl.utils.platform import platform as fslplatform
-import fsleyes_props                      as props
+import fsleyes_widgets                    as fwidgets
 
 
 log = logging.getLogger(__name__)
@@ -200,7 +199,7 @@ if not fslplatform.canHaveGui:
     os.environ['PYOPENGL_PLATFORM'] = 'osmesa'
 
 
-import OpenGL
+import OpenGL  # noqa
 
 
 # Make PyOpenGL throw an error, instead of implicitly
@@ -265,6 +264,9 @@ def bootstrap(glVersion=None):
     ``glmesh_funcs``       The version-specific module containing functions for
                            rendering :class:`.GLMesh` instances.
 
+    ``glmask_funcs``       The version-specific module containing functions for
+                           rendering :class:`.GLMask` instances.
+
     ``gllabel_funcs``      The version-specific module containing functions for
                            rendering :class:`.GLLabel` instances.
 
@@ -314,7 +316,7 @@ def bootstrap(glVersion=None):
         verstr = '1.4'
         glpkg  = gl14
     else: raise RuntimeError('OpenGL 1.4 or newer is required '
-                             '(detected version: {:0.1f}'.format(glVersion))
+                             '(detected version: {:0.1f})'.format(glVersion))
 
     # The gl21 implementation depends on a
     # few extensions - if they're not present,
@@ -362,8 +364,10 @@ def bootstrap(glVersion=None):
     log.debug('Using OpenGL {} implementation with renderer {}'.format(
         verstr, renderer))
 
-    # If on linux, we need to call glutInit.
-    # If on OSX, we don't need to bother.
+    # If using freeglut, we need to call
+    # glutInit. If not (e.g. on mac, which
+    # provides its own GLUT implementation),
+    # we don't need to bother.
     #
     # note: GLUT is only required for text
     #       rendering. The GLUT requirement
@@ -374,8 +378,11 @@ def bootstrap(glVersion=None):
     #       as a requirement. Off-screen
     #       rendering is possible via other
     #       means (e.g. xvfb-run).
-    if fslplatform.os == 'Linux':
-        import OpenGL.GLUT as GLUT
+
+    import OpenGL.GLUT as GLUT
+    # it would be nice if there were a
+    # nicer way to test for freeglut
+    if bool(GLUT.fgDeinitialize):
         GLUT.glutInit()
 
     # Populate this module, and set
@@ -385,6 +392,7 @@ def bootstrap(glVersion=None):
     thismod.glvolume_funcs     = glpkg.glvolume_funcs
     thismod.glrgbvector_funcs  = glpkg.glrgbvector_funcs
     thismod.gllinevector_funcs = glpkg.gllinevector_funcs
+    thismod.glmask_funcs       = glpkg.glmask_funcs
     thismod.glmesh_funcs       = glpkg.glmesh_funcs
     thismod.gllabel_funcs      = glpkg.gllabel_funcs
     thismod.gltensor_funcs     = glpkg.gltensor_funcs
@@ -619,7 +627,7 @@ class GLContext(object):
             # above).  If an existing wx.App is running,
             # we just schedule the context creation
             # routine on it.
-            async.idle(create, alwaysQueue=True)
+            idle.idle(create, alwaysQueue=True)
 
             if self.__ownApp:
                 log.debug('Starting temporary wx.MainLoop')
@@ -684,7 +692,7 @@ class GLContext(object):
         # though. This does mean that we don't have
         # control over depth/stencil buffer sizes,
         # under these remote desktop environments.
-        except:
+        except Exception:
             self.__canvas = wxgl.GLCanvas(self.__parent)
             self.__canvas.SetSize((0, 0))
 
@@ -794,6 +802,11 @@ class OffScreenCanvasTarget(object):
         return self.__width, self.__height
 
 
+    def GetScaledSize(self):
+        """Returns a tuple containing the canvas width and height."""
+        return self.GetSize()
+
+
     def Refresh(self, *a):
         """Does nothing. This canvas is for static (i.e. unchanging) rendering.
         """
@@ -823,6 +836,12 @@ class OffScreenCanvasTarget(object):
         pass
 
 
+    def EnableHighDPI(self):
+        """Does nothing. This canvas is for static (i.e. unchanging) rendering.
+        """
+        pass
+
+
     def draw(self):
         """Calls the :meth:`_draw` method, which must be provided by
         subclasses.
@@ -857,23 +876,18 @@ class OffScreenCanvasTarget(object):
 
 
 WXGLMetaClass = None
-"""Under Python3/wxPython-Phoenix, we must specify a meta-class which derives
-from the ``wx.siplib.wrappertype``, and from the :class:`.props.PropertyOwner`
-meta-class.  This is not necessary under Python2/wxPython.
+"""Under Python3/wxPython-Phoenix, we must specify ``wx.siplib.wrappertype``
+as the meta-class.  This is not necessary under Python2/wxPython.
 """
 
 
 if fslplatform.wxFlavour == fslplatform.WX_PHOENIX:
 
     import wx.siplib as sip
-
-    class PhoenixMeta(props.PropertyOwner, sip.wrappertype):
-        pass
-
-    WXGLMetaClass = PhoenixMeta
+    WXGLMetaClass = sip.wrappertype
 
 else:
-    WXGLMetaClass = props.PropertyOwner
+    WXGLMetaClass = type
 
 
 class WXGLCanvasTarget(object):
@@ -914,6 +928,7 @@ class WXGLCanvasTarget(object):
         self.__glReady           = False
         self.__freezeDraw        = False
         self.__freezeSwapBuffers = False
+        self.__dpiscale          = 1.0
         self.__context           = context
 
         self.Bind(wx.EVT_PAINT,            self.__onPaint)
@@ -931,14 +946,14 @@ class WXGLCanvasTarget(object):
         """
 
         def doRefresh():
-            if fslplatform.isWidgetAlive(self):
+            if fwidgets.isalive(self):
                 self.Refresh()
 
         # GL canvases do need to be refreshed
         # on EVT_PAINT events. If they are not,
         # the canvas will be corrupted.
         if not self.__freezeDraw:
-            async.idle(doRefresh)
+            idle.idle(doRefresh)
 
 
     def _initGL(self):
@@ -1052,6 +1067,35 @@ class WXGLCanvasTarget(object):
         return self.GetClientSize().Get()
 
 
+    def GetScale(self):
+        """Returns the current DPI scaling factor. """
+
+        # GetContentScaleFactor does not
+        # exist in wxpython 3.0.2.0
+        try:
+            scale = float(self.GetContentScaleFactor())
+        except AttributeError:
+            scale = 1.0
+
+        # Reset scaling factor in case the canvas
+        # window has moved between displays with
+        # different scaling factors
+        if scale == 1 and self.__dpiscale != 1:
+            self.EnableHighDPI(False)
+
+        return self.__dpiscale
+
+
+    def GetScaledSize(self):
+        """Returns the current canvas size, scaled by the current DPI scaling
+        factor.
+        """
+        w, h = self.GetSize()
+        s    = self.GetScale()
+
+        return int(round(w * s)), int(round(h * s))
+
+
     def Refresh(self, *a):
         """Triggers a redraw via the :meth:`_draw` method. """
         self.__realDraw()
@@ -1090,16 +1134,52 @@ class WXGLCanvasTarget(object):
                 super(WXGLCanvasTarget, self).SwapBuffers()
 
 
+    def EnableHighDPI(self, enable=True):
+        """Attempts to enable/disable high-resolution rendering.
+        """
+
+        if not self._setGLContext():
+            return
+
+        self.__dpiscale = 1.0
+
+        # GetContentScaleFactor does not
+        # exist in wxpython 3.0.2.0
+        try:
+            scale = self.GetContentScaleFactor()
+
+        except AttributeError:
+            return
+
+        # If the display can't scale,
+        # (scale == 1) there's no point
+        # in enabling it.
+        scale  = float(scale)
+        enable = enable and scale > 1
+
+        # TODO Support other platforms
+        try:
+            import objc
+        except ImportError:
+            return
+
+        nsview = objc.objc_object(c_void_p=self.GetHandle())
+        nsview.setWantsBestResolutionOpenGLSurface_(enable)
+
+        if enable: self.__dpiscale = scale
+        else:      self.__dpiscale = 1.0
+
+
     def getBitmap(self):
         """Return a (width*height*4) shaped numpy array containing the
         rendered scene as an RGBA bitmap.
         """
-        import OpenGL.GL        as gl
-        import numpy            as np
+        import OpenGL.GL as gl
+        import numpy     as np
 
         self._setGLContext()
 
-        width, height = self.GetSize()
+        width, height = self.GetScaledSize()
 
         # Make sure we're reading
         # from the front buffer
@@ -1111,7 +1191,7 @@ class WXGLCanvasTarget(object):
             gl.GL_RGBA,
             gl.GL_UNSIGNED_BYTE)
 
-        bmp = np.fromstring(bmp, dtype=np.uint8)
+        bmp = np.frombuffer(bmp, dtype=np.uint8)
         bmp = bmp.reshape((height, width, 4))
         bmp = np.flipud(bmp)
 
