@@ -8,11 +8,12 @@
 types in an OpenGL 1.4 compatible manner.
 """
 
-from . import glvolume_funcs
-from . import glrgbvector_funcs
-from . import gllinevector_funcs
-from . import glmesh_funcs
-from . import gllabel_funcs
+from . import glvolume_funcs      # noqa
+from . import glrgbvector_funcs   # noqa
+from . import gllinevector_funcs  # noqa
+from . import glmesh_funcs        # noqa
+from . import glmask_funcs        # noqa
+from . import gllabel_funcs       # noqa
 
 gltensor_funcs = None
 glsh_funcs     = None
