@@ -275,10 +275,10 @@ class Texture2D(Texture):
     def getDataTypeParams(cls, dtype):
         """Returns a tuple containing information about the given sized
         internal GL texture data format:
-          - The base GL internal format
-          - The GL external data format
-          - The equivalent ``numpy`` data type
-          - The number of channels
+        - The base GL internal format
+        - The GL external data format
+        - The equivalent ``numpy`` data type
+        - The number of channels
         """
 
         if dtype == gl.GL_RGBA8:
@@ -296,6 +296,13 @@ class Texture2D(Texture):
         return intFmt, extFmt, ndtype, size
 
 
+    @property
+    def dtype(self):
+        """Returns the internal GL data format to use for this texture. """
+
+        return self.__dtype
+
+
     def getSize(self):
         """Return the current ``(width, height)`` of this ``Texture2D``. """
         return self.__width, self.__height
@@ -329,7 +336,7 @@ class Texture2D(Texture):
         if not bound:
             self.unbindTexture()
 
-        data = np.fromstring(data, dtype=ndtype)
+        data = np.frombuffer(data, dtype=ndtype)
         data = data.reshape((self.__height, self.__width, size))
         data = np.flipud(data)
 
@@ -425,51 +432,85 @@ class Texture2D(Texture):
         self.unbindTexture()
 
 
-    def draw(self, vertices, xform=None):
-        """Draw the contents of this ``Texture2D`` to a region specified by
-        the given vertices. The texture is bound to texture unit 0.
+    def __prepareCoords(self, vertices, xform=None):
+        """Called by :meth:`draw`. Prepares vertices, texture coordinates and
+        indices for drawing the texture.
 
-        :arg vertices: A ``numpy`` array of shape ``6 * 3`` specifying the
-                       region, made up of two triangles, to which this
-                       ``Texture2D`` should be drawn.
+        If ``vertices is None``, it is assumed that the caller has already
+        assigned vertices and texture coordinates, either via a shader, or
+        via vertex/texture coordinate pointers. In this case,
 
-        :arg xform:    A transformation to be applied to the vertices.
+        :returns: A tuple containing the vertices, texture coordinates, and
+                  indices, or ``(None, None, indices)`` if
+                  ``vertices is None``
         """
 
+        indices = np.arange(6, dtype=np.uint32)
+
+        if vertices is None:
+            return None, None, indices
+
         if vertices.shape != (6, 3):
             raise ValueError('Six vertices must be provided')
 
         if xform is not None:
             vertices = transform.transform(vertices, xform)
 
-        vertices  = np.array(vertices, dtype=np.float32)
-        texCoords = np.zeros((6, 2),   dtype=np.float32)
-        indices   = np.arange(6,       dtype=np.uint32)
+        vertices  = np.array(vertices, dtype=np.float32).ravel('C')
+        texCoords = self.generateTextureCoords()        .ravel('C')
 
-        texCoords[0, :] = [0, 0]
-        texCoords[1, :] = [0, 1]
-        texCoords[2, :] = [1, 0]
-        texCoords[3, :] = [1, 0]
-        texCoords[4, :] = [0, 1]
-        texCoords[5, :] = [1, 1]
+        return vertices, texCoords, indices
 
-        vertices  = vertices .ravel('C')
-        texCoords = texCoords.ravel('C')
 
-        self.bindTexture(gl.GL_TEXTURE0)
+    def draw(self,
+             vertices=None,
+             xform=None,
+             textureUnit=None):
+        """Draw the contents of this ``Texture2D`` to a region specified by
+        the given vertices. The texture is bound to texture unit 0.
 
-        gl.glClientActiveTexture(gl.GL_TEXTURE0)
+        :arg vertices:    A ``numpy`` array of shape ``6 * 3`` specifying the
+                          region, made up of two triangles, to which this
+                          ``Texture2D`` should be drawn. If ``None``, it is
+                          assumed that the vertices and texture coordinates
+                          have already been configured (e.g. via a shader
+                          program).
 
+        :arg xform:       A transformation to be applied to the vertices.
+                          Ignored if ``vertices is None``.
+
+        :arg textureUnit: Texture unit to bind to. Defaults to
+                          ``gl.GL_TEXTURE0``.
+        """
+
+        if textureUnit is None:
+            textureUnit = gl.GL_TEXTURE0
+
+        vertices, texCoords, indices = self.__prepareCoords(vertices, xform)
+
+        self.bindTexture(textureUnit)
+        gl.glClientActiveTexture(textureUnit)
         gl.glTexEnvf(gl.GL_TEXTURE_ENV,
                      gl.GL_TEXTURE_ENV_MODE,
                      gl.GL_REPLACE)
 
-        with glroutines.enabled((gl.GL_TEXTURE_2D,
-                                 gl.GL_TEXTURE_COORD_ARRAY,
-                                 gl.GL_VERTEX_ARRAY)):
 
-            gl.glVertexPointer(  3, gl.GL_FLOAT, 0, vertices)
-            gl.glTexCoordPointer(2, gl.GL_FLOAT, 0, texCoords)
+        glfeatures = [gl.GL_TEXTURE_2D, gl.GL_VERTEX_ARRAY]
+
+        # Only enable texture coordinates if we know
+        # that there are texture coordinates. Some GL
+        # platforms will crash if texcoords are
+        # enabled on a texture unit, but no texcoords
+        # are loaded.
+        if vertices is not None:
+            glfeatures.append(gl.GL_TEXTURE_COORD_ARRAY)
+
+        with glroutines.enabled(glfeatures):
+
+            if vertices is not None:
+                gl.glVertexPointer(  3, gl.GL_FLOAT, 0, vertices)
+                gl.glTexCoordPointer(2, gl.GL_FLOAT, 0, texCoords)
+
             gl.glDrawElements(gl.GL_TRIANGLES, 6, gl.GL_UNSIGNED_INT, indices)
 
         self.unbindTexture()
@@ -503,6 +544,31 @@ class Texture2D(Texture):
         All other arguments are passed to the :meth:`draw` method.
         """
 
+        vertices = self.generateVertices(
+            zpos, xmin, xmax, ymin, ymax, xax, yax)
+        self.draw(vertices, *args, **kwargs)
+
+
+    @classmethod
+    def generateVertices(
+            cls, zpos, xmin, xmax, ymin, ymax, xax, yax, xform=None):
+        """Generates a set of vertices suitable for passing to the
+        :meth:`.Texture2D.draw` method, for drawing a ``Texture2D`` to a 2D
+        canvas.
+
+        :arg zpos:  Position along the Z axis, in the display coordinate
+                    system.
+        :arg xmin:  Minimum X axis coordinate.
+        :arg xmax:  Maximum X axis coordinate.
+        :arg ymin:  Minimum Y axis coordinate.
+        :arg ymax:  Maximum Y axis coordinate.
+        :arg xax:   Display space axis which maps to the horizontal screen
+                    axis.
+        :arg yax:   Display space axis which maps to the vertical screen
+                    axis.
+        :arg xform: Transformation matrix to appply to vertices.
+        """
+
         zax              = 3 - xax - yax
         vertices         = np.zeros((6, 3), dtype=np.float32)
         vertices[:, zax] = zpos
@@ -514,4 +580,25 @@ class Texture2D(Texture):
         vertices[ 4, [xax, yax]] = [xmin, ymax]
         vertices[ 5, [xax, yax]] = [xmax, ymax]
 
-        self.draw(vertices, *args, **kwargs)
+        if xform is not None:
+            vertices = transform.transform(vertices, xform)
+
+        return vertices
+
+
+    @classmethod
+    def generateTextureCoords(cls):
+        """Generates a set of texture coordinates for drawing a
+        :class:`Texture2D`. This function is used by the
+        :meth:`Texture2D.draw` method.
+        """
+
+        texCoords       = np.zeros((6, 2), dtype=np.float32)
+        texCoords[0, :] = [0, 0]
+        texCoords[1, :] = [0, 1]
+        texCoords[2, :] = [1, 0]
+        texCoords[3, :] = [1, 0]
+        texCoords[4, :] = [0, 1]
+        texCoords[5, :] = [1, 1]
+
+        return texCoords
