@@ -26,15 +26,27 @@ See the :mod:`fsleyes` package documentation for more details on ``fsleyes``.
 import            os
 import os.path as op
 import            sys
-import            textwrap
+import            signal
 import            logging
+import            textwrap
 
 import wx
+import appnope
 
 from fsl.utils.platform import platform as fslplatform
 
-import                   fsleyes
-import fsleyes.splash as fslsplash
+import                       fsleyes
+import fsleyes.splash     as fslsplash
+import fsleyes.colourmaps as colourmaps
+
+
+# wx.ModalDialogHook does not exist in wxPython < 4
+if fslplatform.wxFlavour == fslplatform.WX_PYTHON:
+    class ModalDialogHook(object):
+        def Register(self):
+            pass
+
+    wx.ModalDialogHook = ModalDialogHook
 
 
 log = logging.getLogger(__name__)
@@ -43,18 +55,53 @@ log = logging.getLogger(__name__)
 class FSLeyesApp(wx.App):
     """FSLeyes-specific sub-class of ``wx.App``. """
 
-    def __init__(self, *args, **kwargs):
+
+    class ModalHook(wx.ModalDialogHook):
+        """Keeps track of any modal dialogs/windows that are opened.
+
+        Modal dialogs can interfere with shutdown, as they run their own event
+        loop. Therefore we keep a reference is kept to all opened modal
+        dialogs, so we can manually shut them down if needed (see the
+        :func:`main` function).
+        """
+
+        def __init__(self, *args, **kwargs):
+            wx.ModalDialogHook.__init__(self, *args, **kwargs)
+            self.modals = set()
+
+        def Enter(self, dlg):
+            self.modals.add(dlg)
+            return wx.ID_NONE
+
+        def Exit(self, dlg):
+            self.modals.remove(dlg)
+
+
+    def __init__(self):
         """Create a ``FSLeyesApp``. """
 
         self.__overlayList = None
         self.__displayCtx  = None
 
-        wx.App.__init__(self, *args, **kwargs)
+        self.__modalHook = FSLeyesApp.ModalHook()
+        self.__modalHook.Register()
+
+        wx.App.__init__(self, clearSigInt=False)
 
         self.SetAppName('FSLeyes')
 
 
+    @property
+    def modals(self):
+        """Returns a list of all currently open modal windows. """
+        return list(self.__modalHook.modals)
+
+
     def SetOverlayListAndDisplayContext(self, overlayList, displayCtx):
+        """References to the :class:`.OverlayList` and master
+        :class:`.DisplayContext` must be passed to the ``FSLeyesApp`` via this
+        method.
+        """
         self.__overlayList = overlayList
         self.__displayCtx  = displayCtx
 
@@ -143,27 +190,77 @@ def main(args=None):
         render.main(args[1:])
         sys.exit(0)
 
-    # Implement various hacks and workarounds
-    hacksAndWorkarounds()
+    # Hook which allows us to run a jupyter
+    # notebook server  within a frozen version
+    # of FSLeyes
+    if len(args) == 2 and args[0] == 'notebook' and fslplatform.frozen:
+        from notebook.notebookapp import main
 
-    # Then, first thing's first. Create a wx.App,
-    # and initialise the FSLeyes package.
-    app = FSLeyesApp()
+        # second argument is a path
+        # to add to the PYTHONPATH.
+        # See fsleyes.actions.notebook.NotebookServer.run.
+        sys.path.insert(0, args[1])
+        fsleyes.configLogging()
+        sys.exit(main(argv=[]))
+
+    # the fsleyes.initialise function figures
+    # out the path to asset files (e.g. cmaps)
     fsleyes.initialise()
 
-    # Show the splash screen as soon as
-    # possible, unless it looks like the
-    # user is asking for the software
-    # version or command line help.
-    splash = fslsplash.FSLeyesSplash(None)
+    # initialise colour maps - this must be
+    # done before parsing arguments, as if
+    # the user asks for help, available
+    # colourmaps/luts will be listed.
+    colourmaps.init()
 
+    # Function to bootstrap the GUI - keep
+    # reading below.
+    def initgui():
+
+        # First thing's first. Create a wx.App,
+        # and initialise the FSLeyes package.
+        app = FSLeyesApp()
+
+        # Create a splash screen frame
+        splash = fslsplash.FSLeyesSplash(None)
+        return app, splash
+
+    # If it looks like the user is asking for
+    # help, then we parse command line arguments
+    # before creating a wx.App and showing the
+    # splash screen. This means that FSLeyes
+    # help/version information can be retrieved
+    # without a display, and hopefully fairly
+    # quickly.
+    #
+    # Otherwise we create the app and splash
+    # screen first, so the splash screen gets
+    # shown as soon as possible. Arguments
+    # will get parsed in the init function below.
+    #
+    # The argparse.Namespace object is kept in a
+    # list so it can be shared between the sub-
+    # functions below
+    #
+    # If argument parsing bombs out, we put the
+    # exit code here and return it at the bottom.
+    namespace = [None]
+    exitCode  = [0]
+
+    # user asking for help - parse args first
     if (len(args) > 0) and (args[0] in ('-V',
                                         '-h',
                                         '-fh',
                                         '--version',
                                         '--help',
                                         '--fullhelp')):
-        splash.Hide()
+        namespace   = [parseArgs(args)]
+        app, splash = initgui()
+
+    # otherwise parse arguments on wx.MainLoop
+    # below
+    else:
+        app, splash = initgui()
 
     # We are going do all processing on the
     # wx.MainLoop, so the GUI can be shown
@@ -180,31 +277,22 @@ def main(args=None):
     # via ugly callbacks, but which are
     # ultimately scheduled and executed on the
     # wx main loop.
-
-    # This is a container, shared amongst
-    # the callbacks, which contains the
-    # parsed argparse.Namespace object.
-    namespace = [None]
-
-    # If argument parsing bombs out,
-    # we put the exit code here and
-    # return it at the bottom.
-    exitCode = [0]
-
     def init(splash):
 
-        # Parse command line arguments. If the
-        # user has asked for help (see above),
-        # the parseargs module will raise
-        # SystemExit. Hence we make sure the
-        # splash screen is shown only after
-        # arguments have been parsed.
+        # See FSLeyesSplash.Show
+        # for horribleness.
+        splash.Show()
+
+        # Parse command line arguments if necessary.
+        # If arguments are invalid, the parseargs
+        # module will raise SystemExit.
         try:
-            namespace[0] = parseArgs(args)
+            if namespace[0] is None:
+                namespace[0] = parseArgs(args)
 
         # But the wx.App.MainLoop eats SystemExit
         # exceptions for unknown reasons, and
-        # and causes the application to exit
+        # causes the application to exit
         # immediately. This makes testing FSLeyes
         # (e.g. code coverage) impossible. So I'm
         # catching SystemExit here, and then
@@ -214,14 +302,10 @@ def main(args=None):
             exitCode[0] = e.code
             return
 
-        # See FSLeyesSplash.Show
-        # for horribleness.
-        splash.Show()
-
         # Configure logging (this has to be done
         # after cli arguments have been parsed,
         # but before initialise is called).
-        fsleyes.configLogging(namespace[0])
+        fsleyes.configLogging(namespace[0].verbose, namespace[0].noisy)
 
         # Initialise sub-modules/packages. The
         # buildGui function is passed through
@@ -249,12 +333,40 @@ def main(args=None):
         # Check for updates. Ignore point
         # releases, otherwise users might
         # get swamped with update notifications.
-        if not namespace[0].skipupdatecheck:
+        if namespace[0].updatecheck:
             import fsleyes.actions.updatecheck as updatecheck
             wx.CallAfter(updatecheck.UpdateCheckAction(),
                          showUpToDateMessage=False,
                          showErrorMessage=False,
-                         ignorePoint=True)
+                         ignorePoint=False)
+
+    # Shut down cleanly on sigint/sigterm.
+    # We do this so that any functions
+    # registered with atexit will actually
+    # get called.
+    nsignals = [0]
+
+    def sigHandler(signo, frame):
+        log.debug('Signal received - FSLeyes is shutting down...')
+
+        # first signal - try to exit cleanly
+        if nsignals[0] == 0:
+            nsignals[0] += 1
+            exitCode[0]  = signo
+
+            # kill any modal windows
+            # that are open
+            for mdlg in app.modals:
+                mdlg.EndModal(wx.ID_CANCEL)
+
+            wx.CallAfter(app.ExitMainLoop)
+
+        # subsequent signals - exit immediately
+        else:
+            sys.exit(signo)
+
+    signal.signal(signal.SIGINT,  sigHandler)
+    signal.signal(signal.SIGTERM, sigHandler)
 
     # Note: If no wx.Frame is created, the
     # wx.MainLoop call will exit immediately,
@@ -263,54 +375,12 @@ def main(args=None):
     # already created the splash screen, so
     # all is well.
     wx.CallAfter(init, splash)
+    appnope.nope()
     app.MainLoop()
     shutdown()
     return exitCode[0]
 
 
-def hacksAndWorkarounds():
-    """Called by :func:`main`. Implements hacks and workarounds for
-    various things.
-    """
-
-    # Under wxPython/Phoenix, the
-    # wx.html package must be imported
-    # before a wx.App has been created
-    import wx.html
-
-    # PyInstaller 3.2.1 forces matplotlib to use a
-    # temporary directory for its settings and font
-    # cache, and then deletes the directory on exit.
-    # This is silly, because the font cache can take
-    # a long time to create.  Clearing the environment
-    # variable should cause matplotlib to use
-    # $HOME/.matplotlib (or, failing that, a temporary
-    # directory).
-    #
-    # https://matplotlib.org/faq/environment_variables_faq.html#\
-    #   envvar-MPLCONFIGDIR
-    #
-    # https://github.com/pyinstaller/pyinstaller/blob/v3.2.1/\
-    #   PyInstaller/loader/rthooks/pyi_rth_mplconfig.py
-    #
-    # n.b. This will cause issues if building FSLeyes
-    #      with the pyinstaller '--onefile' option, as
-    #      discussed in the above pyinstaller file.
-    if fslplatform.frozen:
-        os.environ.pop('MPLCONFIGDIR', None)
-
-    # OSX sometimes sets the local environment
-    # variables to non-standard values, which
-    # breaks the python locale module.
-    #
-    # http://bugs.python.org/issue18378
-    try:
-        import locale
-        locale.getdefaultlocale()
-    except:
-        os.environ['LC_ALL'] = 'C.UTF-8'
-
-
 def initialise(splash, namespace, callback):
     """Called by :func:`main`. Bootstraps/Initialises various parts of
     *FSLeyes*.
@@ -329,12 +399,9 @@ def initialise(splash, namespace, callback):
     import fsl.utils.settings as fslsettings
     import fsleyes_props      as props
     import fsleyes.gl         as fslgl
-    import fsleyes.colourmaps as colourmaps
 
     props.initGUI()
 
-    colourmaps.init()
-
     # The save/load directory defaults
     # to the current working directory.
     curDir = op.normpath(os.getcwd())
@@ -355,7 +422,7 @@ def initialise(splash, namespace, callback):
         #   [install_dir]/FSLeyes.app/Contents/MacOS/fsleyes/),
         #
         # Because the cwd will default to:
-        #   [install_dir/
+        #   [install_dir]/
 
         if fslplatform.os == 'Darwin':
 
@@ -373,10 +440,11 @@ def initialise(splash, namespace, callback):
 
     # Initialise silly things
     if namespace.bumMode:
-        import fsleyes.controls.orthotoolbar    as ot
-        import fsleyes.controls.lightboxtoolbar as lbt
-        ot .BUM_MODE = True
-        lbt.BUM_MODE = True
+        import fsleyes.icons as icons
+        icons.BUM_MODE = True
+
+    # Set notebook server port
+    fslsettings.write('fsleyes.notebook.port', namespace.notebookPort)
 
     # This is called by fsleyes.gl.getGLContext
     # when the GL context is ready to be used.
@@ -391,7 +459,7 @@ def initialise(splash, namespace, callback):
         # canvas created by the gl.getWXGLContext function.
         fslgl.getGLContext(parent=splash, ready=realCallback)
 
-    except:
+    except Exception:
         log.error('Unable to initialise OpenGL!', exc_info=True)
         splash.Destroy()
         sys.exit(1)
@@ -418,9 +486,9 @@ def parseArgs(argv):
     :arg argv: command line arguments for ``fsleyes``.
     """
 
-    import fsleyes.parseargs    as parseargs
-    import fsleyes.perspectives as perspectives
-    import fsleyes.version      as version
+    import fsleyes.parseargs as parseargs
+    import fsleyes.layouts   as layouts
+    import fsleyes.version   as version
 
     parser = parseargs.ArgumentParser(
         add_help=False,
@@ -431,21 +499,21 @@ def parseArgs(argv):
                         help='Run custom FSLeyes script')
 
     # We include the list of available
-    # perspectives in the help description
-    persps      = list(perspectives.BUILT_IN_PERSPECTIVES.keys()) + \
-                  list(perspectives.getAllPerspectives())
+    # layouts in the help description
+    allLayouts  = list(layouts.BUILT_IN_LAYOUTS.keys()) + \
+                  list(layouts.getAllLayouts())
     name        = 'fsleyes'
     prolog      = 'FSLeyes version {}\n'.format(version.__version__)
     description = textwrap.dedent("""\
         FSLeyes - the FSL image viewer.
 
-        Use the '--scene' option to load a saved perspective ({persps}).
+        Use the '--scene' option to load a saved layout ({layouts}).
 
         If no '--scene' is specified, a default layout is shown or the
         previous layout is restored. If a script is provided via
         the '--runscript' argument, it is assumed that the script sets
         up the scene.
-        """.format(persps=', '.join(persps)))
+        """.format(layouts=', '.join(allLayouts)))
 
     # Options for configuring the scene are
     # managed by the parseargs module
@@ -502,29 +570,12 @@ def makeDisplayContext(namespace, splash):
     overlayList = fsloverlay.OverlayList()
     displayCtx  = displaycontext.DisplayContext(overlayList)
 
-    # While the DisplayContext may refer to
-    # multiple overlay groups, we are currently
-    # using just one, allowing the user to specify
-    # a set of overlays for which their display
-    # properties are 'locked'.
-    lockGroup   = displaycontext.OverlayGroup(displayCtx, overlayList)
-    displayCtx.overlayGroups.append(lockGroup)
-
     log.debug('Created overlay list and master DisplayContext ({})'.format(
         id(displayCtx)))
 
     # Load the images - the splash screen status will
     # be updated with the currently loading overlay name.
-    #
-    # The applyOverlayArgs function gets called before
-    # the applySceneArgs function, so we'll manually
-    # apply some important settings to the DC here so
-    # they get used when any overlays are loaded.
-    if namespace.bigmem is not None:
-        displayCtx.loadInMemory = namespace.bigmem
-    if namespace.autoDisplay is not None:
-        displayCtx.autoDisplay = namespace.autoDisplay
-
+    parseargs.applyMainArgs(   namespace, overlayList, displayCtx)
     parseargs.applyOverlayArgs(namespace, overlayList, displayCtx)
 
     return overlayList, displayCtx
@@ -557,7 +608,7 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
     :returns: the :class:`.FSLeyesFrame` that was created.
     """
 
-    import fsl.utils.async              as async
+    import fsl.utils.idle               as idle
     import fsleyes_widgets.utils.status as status
     import fsleyes.parseargs            as parseargs
 
@@ -566,17 +617,16 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
     # FSLeyesFrame class, so it must be
     # imported immediately after fsleyes.frame.
     import fsleyes.frame                as fsleyesframe
-    import fsleyes.actions.frameactions as frameactions
+    import fsleyes.actions.frameactions as frameactions  # noqa
 
     import fsleyes.displaycontext       as fsldisplay
-    import fsleyes.perspectives         as perspectives
+    import fsleyes.layouts              as layouts
     import fsleyes.views.canvaspanel    as canvaspanel
-    import fsleyes.views.orthopanel     as orthopanel
 
-    # Set up the frame scene (a.k.a. layout, perspective)
+    # Set up the frame scene (a.k.a. layout)
     # The scene argument can be:
     #
-    #   - The name of a saved (or built-in) perspective
+    #   - The name of a saved (or built-in) layout
     #
     #   - None, in which case the default or previous
     #     layout is restored, unless a custom script
@@ -584,7 +634,7 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
     script = namespace.runscript
     scene  = namespace.scene
 
-    # If a scene/perspective or custom script
+    # If a scene/layout or custom script
     # has not been specified, the default
     # behaviour is to restore the previous
     # frame layout.
@@ -593,7 +643,12 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
     status.update('Creating FSLeyes interface...')
 
     frame = fsleyesframe.FSLeyesFrame(
-        None, overlayList, displayCtx, restore, True)
+        None,
+        overlayList,
+        displayCtx,
+        restore,
+        True,
+        fontSize=namespace.fontSize)
 
     # Make sure the new frame is shown
     # before destroying the splash screen
@@ -614,7 +669,12 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
 
     def onFrameDestroy(ev):
         ev.Skip()
-        splash.Close()
+
+        # splash screen may already
+        # have been destroyed
+        try:              splash.Close()
+        except Exception: pass
+
     frame.Bind(wx.EVT_WINDOW_DESTROY, onFrameDestroy)
 
     status.update('Setting up scene...')
@@ -626,20 +686,20 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
         fsldisplay.SceneOpts.performance.setAttribute(
             None, 'default', namespace.performance)
 
-    # If a perspective has been specified,
-    # we load the perspective
+    # If a layout has been specified,
+    # we load the layout
     if namespace.scene is not None:
-        perspectives.loadPerspective(frame, namespace.scene)
+        layouts.loadLayout(frame, namespace.scene)
 
     # Apply any view-panel specific arguments
-    viewPanels = frame.getViewPanels()
+    viewPanels = frame.viewPanels
     for viewPanel in viewPanels:
 
         if not isinstance(viewPanel, canvaspanel.CanvasPanel):
             continue
 
-        displayCtx = viewPanel.getDisplayContext()
-        viewOpts   = viewPanel.getSceneOptions()
+        displayCtx = viewPanel.displayCtx
+        viewOpts   = viewPanel.sceneOpts
 
         parseargs.applySceneArgs(
             namespace, overlayList, displayCtx, viewOpts)
@@ -653,7 +713,7 @@ def makeFrame(namespace, displayCtx, overlayList, splash):
     # script can assume that all overlays have
     # already been loaded.
     if script is not None:
-        async.idle(frame.runScript, script)
+        idle.idle(frame.runScript, script)
 
     return frame
 
