@@ -15,6 +15,8 @@ import numpy              as np
 import matplotlib.patches as patches
 
 import fsleyes_props      as props
+import fsl.utils.idle     as idle
+import fsl.data.image     as fslimage
 
 import fsleyes.overlay    as fsloverlay
 from . import                plotprofile
@@ -58,8 +60,6 @@ class HistogramProfile(plotprofile.PlotProfile):
                                          displayCtx,
                                          ['overlayRange'])
 
-        self.mode = 'overlayRange'
-
         self.__currentHs     = None
         self.__rangePolygons = {}
         self.__rangeOverlays = {}
@@ -74,10 +74,10 @@ class HistogramProfile(plotprofile.PlotProfile):
         self.__draggingRange = False
 
         overlayList.addListener('overlays',
-                                self._name,
+                                self.name,
                                 self.__overlayListChanged)
         displayCtx .addListener('selectedOverlay',
-                                self._name,
+                                self.name,
                                 self.__selectedOverlayChanged)
 
         self.__selectedOverlayChanged()
@@ -88,8 +88,8 @@ class HistogramProfile(plotprofile.PlotProfile):
         Removes property listeners, and cleans some things up.
         """
 
-        self._overlayList.removeListener('overlays',        self._name)
-        self._displayCtx .removeListener('selectedOverlay', self._name)
+        self.overlayList.removeListener('overlays',        self.name)
+        self.displayCtx .removeListener('selectedOverlay', self.name)
 
         for hs in list(self.__rangePolygons.keys()):
             self.__deregisterHistogramSeries(hs)
@@ -106,7 +106,7 @@ class HistogramProfile(plotprofile.PlotProfile):
         """
 
         for hs in list(self.__rangePolygons.keys()):
-            if hs.overlay not in self._overlayList:
+            if hs.overlay not in self.overlayList:
                 self.__deregisterHistogramSeries(hs)
 
         self.__selectedOverlayChanged()
@@ -123,7 +123,7 @@ class HistogramProfile(plotprofile.PlotProfile):
 
         rangePolygon = RangePolygon(
             hs,
-            self._viewPanel,
+            self.viewPanel,
             np.zeros((2, 2)),
             closed=True,
             linewidth=2)
@@ -131,8 +131,8 @@ class HistogramProfile(plotprofile.PlotProfile):
         rangeOverlay = HistogramOverlay(
             hs,
             hs.overlay,
-            self._displayCtx,
-            self._overlayList)
+            self.displayCtx,
+            self.overlayList)
 
         self.__rangePolygons[hs] = rangePolygon
         self.__rangeOverlays[hs] = rangeOverlay
@@ -157,16 +157,18 @@ class HistogramProfile(plotprofile.PlotProfile):
         exist for the newly selected overlay.
         """
 
-        overlay = self._displayCtx.getSelectedOverlay()
+        overlay = self.displayCtx.getSelectedOverlay()
         oldHs   = self.__currentHs
-        newHs   = self._viewPanel.getDataSeries(overlay)
+        newHs   = self.viewPanel.getDataSeries(overlay)
 
         if oldHs == newHs:
             return
 
-        self.__currentHs = newHs
-
-        self.__registerHistogramSeries(newHs)
+        if isinstance(overlay, fslimage.Image):
+            self.__currentHs = newHs
+            self.__registerHistogramSeries(newHs)
+        else:
+            self.__currentHs = None
 
 
     def __updateShowOverlayRange(self, datax, which=False):
@@ -301,16 +303,16 @@ class RangePolygon(patches.Polygon):
         """
         patches.Polygon.__init__(self, *args, **kwargs)
 
-
         # Dodgily trying to avoid collisions
         # with any patches.Polygon attributes
         self._rp_hs      = hs
         self._rp_hsPanel = hsPanel
         self._rp_name    = '{}_{}'.format(type(self).__name__, id(self))
 
-        hs.addGlobalListener(           self._rp_name, self.updatePolygon)
-        hsPanel.addListener('smooth',   self._rp_name, self.updatePolygon)
-        hsPanel.addListener('histType', self._rp_name, self.updatePolygon)
+        hs.addGlobalListener(           self._rp_name, self.asyncUpdatePolygon)
+        hsPanel.addListener('smooth',   self._rp_name, self.asyncUpdatePolygon)
+        hsPanel.addListener('histType', self._rp_name, self.asyncUpdatePolygon)
+        hsPanel.addListener('plotType', self._rp_name, self.asyncUpdatePolygon)
 
         self.updatePolygon()
 
@@ -327,6 +329,7 @@ class RangePolygon(patches.Polygon):
         hs.removeGlobalListener(           self._rp_name)
         hsPanel.removeListener('smooth',   self._rp_name)
         hsPanel.removeListener('histType', self._rp_name)
+        hsPanel.removeListener('plotType', self._rp_name)
 
         if self in hsPanel.artists:
             hsPanel.artists.remove(self)
@@ -335,6 +338,46 @@ class RangePolygon(patches.Polygon):
         self._rp_hsPanel = None
 
 
+    def asyncUpdatePolygon(self, *a, **kwa):
+        """Asynchronously schedule :meth:`updatePolygon` via
+        :func:`.idle.idle`.
+        """
+
+        # We need to make sure, if the histogram
+        # data has changed, that the data plot is
+        # updated before the polygon is updated.
+        #
+        # This is subtly complicated, and is tightly
+        # coupled to the OverlayPlotPanel internals.
+        #
+        # 1. OverlayPlotPanel draws via asyncDraw(),
+        #    which schedules draw() on the idle loop
+        #    (the idle.idle call below makes sure
+        #     we get scheduled after this).
+        #
+        # 2. draw() prepares data on a TaskThread
+        #    (the OPP draw queue; waitUntilIdle
+        #     is run on a separate thread which
+        #     blocks until the draw queue is empty).
+        #
+        # 3. When data is prepared, __drawDataSeries()
+        #    is scheduled on idle loop (the onFinish
+        #    function below - updatePolygon - is
+        #    schduled on idle after this).
+        #
+        # We need to wait until all of the above has
+        # completed before updating the range polygon.
+
+        q        = self._rp_hsPanel.getDrawQueue()
+        taskName = '{}.draw'.format(id(self))
+
+        idle.idle(idle.run,
+                  q.waitUntilIdle,
+                  onFinish=self.updatePolygon,
+                  name=taskName,
+                  dropIfQueued=True)
+
+
     def updatePolygon(self, *a, **kwa):
         """Called whenever any property changes on the
         :class:`.HistogramSeries`, and called manually by the
@@ -347,15 +390,17 @@ class RangePolygon(patches.Polygon):
         hs      = self._rp_hs
         hsPanel = self._rp_hsPanel
 
-        # If smoothing is enabled, we get
+        # If smoothing is enabled, or we are
+        # plotting bin centres, we get
         # the histogram data from the plotted
         # Line2D instance. This is because the
-        # HistogramPanel does the smoothing,
-        # not the HistogramSeries instance.
+        # HistogramPanel does the smoothing and
+        # bin centre adjustmennt, not the
+        # HistogramSeries instance.
         #
         # Try/except because the HistogramSeries
         # may not yet have been plotted.
-        if hsPanel.smooth:
+        if hsPanel.smooth or hsPanel.plotType == 'centre':
             try:
 
                 hsArtist = hsPanel.getArtist(hs)
@@ -363,7 +408,7 @@ class RangePolygon(patches.Polygon):
                 y        = hsArtist.get_ydata()
                 vertices = np.array([x, y]).T
 
-            except:
+            except Exception:
                 vertices = np.zeros(0)
 
         # Otherwise we can get the data
@@ -398,8 +443,8 @@ class RangePolygon(patches.Polygon):
 
             histx, histy = hs.getData()
 
-            try:    xidx = np.where(histx[:] < hi)[0][-1]
-            except: xidx = len(histy) - 1
+            try:              xidx = np.where(histx[:] < hi)[0][-1]
+            except Exception: xidx = len(histy) - 1
 
             if xidx >= len(histy):
                 xidx = -1
