@@ -24,7 +24,7 @@ into the following categories:
  :data:`nifti`        Labels for NIFTI header fields.
  :data:`feat`         FEAT specific names and labels.
  :data:`melodic`      MELODIC specific names and labels.
- :data:`perspectives` Perspective labels.
+ :data:`layouts`      Layout labels.
  :data:`tensor`       Tensor overlay labels.
  :data:`plotLabels`   Labels to use for plot data loaded from known files.
  :data:`about`        Strings used in the *FSLeyes* about dialog.
@@ -49,20 +49,19 @@ messages = TypeDict({
     'FSLeyesFrame.unsavedOverlays'     : 'You have unsaved images - are '
                                          'you sure you want to exit?',
 
-    'perspectives.applyingPerspective' : 'Applying {} perspective ...',
+    'layout.applyingLayout' : 'Applying {} layout ...',
 
-    'SavePerspectiveAction.enterName'        : 'Enter a name for the '
-                                               'perspective',
-    'SavePerspectiveAction.nameIsBuiltIn'    : '"{}" is a reserved '
-                                               'perspective name - '
-                                               'enter a different name.',
-    'SavePerspectiveAction.confirmOverwrite' : 'A perspective with the name '
-                                               '"{}" already exists - do '
-                                               'you want to replace it?',
+    'SaveLayoutAction.enterName' :
+    'Enter a name for the layout',
+    'SaveLayoutAction.nameIsBuiltIn' :
+    '"{}" is a reserved layout name - enter a different name.',
+    'SaveLayoutAction.confirmOverwrite' :
+    'A layout with the name "{}" already exists - do '
+    'you want to replace it?',
 
-    'ClearPerspectiveAction.confirmClear' : 'All saved perspectives will be '
-                                            'cleared! Are you sure you want '
-                                            'to continue?',
+    'ClearLayoutsAction.confirmClear' : 'All saved layouts will be '
+                                        'cleared! Are you sure you want '
+                                        'to continue?',
 
     'FSLeyesApp.openURLError' : 'An error occurred loading the URL.',
 
@@ -98,6 +97,10 @@ messages = TypeDict({
     'A lookup table with that name already exists - choose a different name.',
     'LookupTablePanel.installerror' :
     'An error occurred while installing the lookup table.',
+    'LookupTablePanel.labelExists' :
+    'The {} LUT already contains a label with value {}',
+    'LookupTablePanel.loadError' :
+    'An error occurred while loading {}',
 
     'actions.copyoverlay.createMask'  : 'Create empty mask image '
                                         'with same dimensions',
@@ -136,9 +139,6 @@ messages = TypeDict({
 
     'HistogramPanel.calcHist'           : 'Calculating histogram for {} ...',
 
-    'LookupTablePanel.labelExists' : 'The {} LUT already contains a '
-                                     'label with value {}',
-
     'ClusterPanel.noOverlays'     : 'Add a FEAT overlay',
     'ClusterPanel.notFEAT'        : 'Choose a FEAT overlay',
     'ClusterPanel.noClusters'     : 'No cluster results exist '
@@ -147,24 +147,6 @@ messages = TypeDict({
                                     'check your cluster_*.txt files.',
     'ClusterPanel.loadingCluster' : 'Loading clusters for COPE{} ({}) ...',
 
-    'OrthoEditProfile.imageChange'        : 'You are now editing {}. ',
-    'OrthoEditProfile.imageChangeHint'    : 'Setting {} as the display '
-                                            'space reference\nimage - the '
-                                            'display space must match the '
-                                            'image being edited.',
-
-    'OrthoEditProfile.imageChange.suppress' : 'Do not show this '
-                                              'message again',
-
-    'OrthoCropProfile.imageChange'        : 'You are now cropping {}. ',
-    'OrthoCropProfile.imageChangeHint'    : 'Setting {} as the display '
-                                            'space reference\nimage - the '
-                                            'display space must match the '
-                                            'image being cropped.',
-
-    'OrthoCropProfile.imageChange.suppress' : 'Do not show this '
-                                              'message again',
-
     'MelodicClassificationPanel.disabled' :
     'Choose a melodic or other 4D image.',
     'MelodicClassificationPanel.loadError' :
@@ -189,6 +171,8 @@ messages = TypeDict({
     'ScreenshotAction.pleaseWait' : 'Saving screenshot to {}...',
     'ScreenshotAction.error'      : 'An error occurred saving the screenshot.',
 
+    'MovieGifAction.movieGif'     : 'Save animated GIF',
+
     'SaveOverlayAction.saveError' :
     'An error occurred while saving the file {}.',
 
@@ -229,7 +213,7 @@ messages = TypeDict({
 
     'ClearSettingsAction.confirm' :
     'Are you sure you want to clear all FSLeyes settings? All preferences, '
-    'saved perspectives, colour maps, lookup tables, and loaded atlases will '
+    'saved layouts, colour maps, lookup tables, and loaded atlases will '
     'be lost!\n\nYou will need to restart FSLeyes for some changes to take '
     'effect.',
 
@@ -237,12 +221,12 @@ messages = TypeDict({
     'Calculating correlation values for seed voxel [{}, {}, {}] ...',
 
     'EditTransformPanel.saveFlirt.error' :
-    'An error occurred saving the FLIRT matrix.',
+    'An error occurred saving the affine matrix.',
 
     'SaveFlirtXfmAction.error' :
-    'An error occurred saving the FLIRT matrix.',
+    'An error occurred saving the affine matrix.',
 
-    'FlirtFileDialog.matFile'    : 'Select FLIRT transformation matrix file',
+    'FlirtFileDialog.matFile'    : 'Select affine transformation matrix file',
     'FlirtFileDialog.refFile'    : 'Select FLIRT reference image',
 
     'CropImagePanel.saveCrop'  : 'Select a file to save the crop parameters',
@@ -251,20 +235,41 @@ messages = TypeDict({
     'An error occurred saving the crop parameters',
     'CropImagePanel.loadError' :
     'An error occurred loading the crop parameters',
+    'CropImagePanel.dsWarning' :
+    'Warning: To crop an image, you must select it\n'
+    'as the display space. You can change the display\n'
+    'space back in the view settings panel.',
 
+    'LocationHistoryPanel.load' : 'Select a location file to load',
+    'LocationHistoryPanel.save' : 'Select a file to save the locations to',
+    'LocationHistoryPanel.loadError' :
+    'An error occurred loading locations from {}',
+    'LocationHistoryPanel.saveError' :
+    'An error occurred saving locations to {}',
+
+    'OrthoEditToolBar.dsWarning' :
+    'Warning: You must set the display\n'
+    'space to the image being edited!',
 
     'LoadVertexDataAction.loadVertexData' :
     'Select a vertex data file for {}',
+
+    'LoadVertexDataAction.loadVertices' :
+    'Select a vertex file for {}',
+
     'LoadVertexDataAction.error' :
     'An error occurred while loading the vertex data for {}',
 
     'UpdateCheckAction.newVersionAvailable' :
-    'A new version of FSLeyes is available. This version of FSLeyes is {}, '
-    'and the latest is {}.\n\nVisit {} to upgrade!',
+    'A new version of FSLeyes is available. This version\n'
+    'of FSLeyes is {}, and the latest is {}.',
 
     'UpdateCheckAction.upToDate' :
     'Your version of FSLeyes ({}) is up to date.',
 
+    'UpdateCheckAction.updateUrl' :
+    'The latest version of FSLeyes is available from:',
+
     'UpdateCheckAction.newVersionError' :
     'An error occurred while checking for FSLeyes updates. Try again later.',
 
@@ -273,6 +278,19 @@ messages = TypeDict({
 
     'ApplyCommandLineAction.error' :
     'An error occurred while applying the command line arguments.',
+
+    'loadDicom.selectDir' : 'Select DICOM directory',
+    'loadDicom.scanning'  : 'Scanning for DICOM data series...',
+    'loadDicom.loading'   : 'Loading DICOM data series...',
+    'loadDicom.scanError' :
+    'An error occurred while scanning the DICOM directory',
+    'loadDicom.loadError' :
+    'An error occurred while loading DICOM data',
+
+    'NotebookAction.init.kernel' : 'Initialising IPython kernel...',
+    'NotebookAction.init.server' : 'Starting Jupyter notebook server...',
+    'NotebookAction.init.error' :
+    'An error occurred initialising IPython/Jupyter! ',
 })
 
 
@@ -317,6 +335,8 @@ titles = TypeDict({
     'LightBoxToolBar'           : 'Lightbox view toolbar',
     'LookupTablePanel'          : 'Lookup tables',
     'LutLabelDialog'            : 'New LUT label',
+    'Scene3DToolBar'            : '3D view toolbar',
+    'FlirtFileDialog'           : 'Load affine transformation',
 
     'PlotListPanel'             : 'Plot list',
     'TimeSeriesControlPanel'    : 'Time series control',
@@ -333,10 +353,14 @@ titles = TypeDict({
     'CropImagePanel'             : 'Crop',
     'EditTransformPanel'         : 'Nudge',
 
+    'LocationHistoryPanel.loadError' : 'Error loading location file',
+    'LocationHistoryPanel.saveError' : 'Error saving location file',
+
     'LookupTablePanel.newlut'       : 'Name lookup table',
     'LookupTablePanel.loadLut'      : 'Select a lookup table file',
     'LookupTablePanel.labelExists'  : 'Label already exists',
     'LookupTablePanel.installerror' : 'Error installing lookup table',
+    'LookupTablePanel.loadError'    : 'Error loading lut file',
 
     'MelodicClassificationPanel.loadDialog' : 'Load FIX/Melview file...',
     'MelodicClassificationPanel.saveDialog' : 'Save FIX/Melview file...',
@@ -345,7 +369,7 @@ titles = TypeDict({
 
     'ScreenshotAction.screenshot'          : 'Save screenshot',
     'ScreenshotAction.error'               : 'Error saving screenshot',
-    'ClearPerspectiveAction.confirmClear'  : 'Clear all perspectives?',
+    'ClearLayoutsAction.confirmClear'      : 'Clear all layouts?',
     'DiagnosticReportAction.saveReport'    : 'Save diagnostic report',
     'SaveOverlayAction.overwrite'          : 'Overwrite existing file?',
     'SaveOverlayAction.saveFile'           : 'Save overlay to file',
@@ -380,7 +404,7 @@ titles = TypeDict({
 
     'LoadVertexDataAction.error' : 'Error loading vertex data',
 
-    'SaveFlirtXfmAction.error' : 'Error saving theFLIRT matrix',
+    'SaveFlirtXfmAction.error' : 'Error saving affine matrix',
 
     'ClearSettingsAction.confirm' : 'Clear all settings?',
 
@@ -395,14 +419,28 @@ titles = TypeDict({
 
     'ApplyCommandLineAction.title' : 'Apply FSLeyes command line',
     'ApplyCommandLineAction.error' : 'Error applying command line',
+
+    'XNATBrowser' : 'Open from XNAT repository',
+
+    'loadDicom.scanning'  : 'Scanning DICOM directory',
+    'loadDicom.loading'   : 'Loading DICOM data series',
+    'loadDicom.scanError' : 'Error reading DICOM directory',
+    'loadDicom.loadError' : 'Error loading DICOM series',
+    'BrowseDicomDialog'   : 'Select DICOM series',
+
+    'NotebookAction.init' : 'Starting Jupyter',
+    'NotebookAction.init.error' :
+    'Error initialising IPython/Jupyter',
 })
 
 
 actions = TypeDict({
 
-    'LoadOverlayAction'        : 'Add overlay from file',
-    'LoadOverlayFromDirAction' : 'Add overlay from directory',
+    'LoadOverlayAction'        : 'Add from file',
+    'LoadOverlayFromDirAction' : 'Add from directory',
     'LoadStandardAction'       : 'Add standard',
+    'LoadDicomAction'          : 'Add from DICOM',
+    'BrowseXNATAction'         : 'Add from XNAT',
     'CopyOverlayAction'        : 'Copy',
     'LoadAtlasAction'          : 'Add atlas',
     'ClearSettingsAction'      : 'Clear FSLeyes settings',
@@ -412,14 +450,15 @@ actions = TypeDict({
     'RemoveOverlayAction'      : 'Remove',
     'RemoveAllOverlaysAction'  : 'Remove all',
     'LoadColourMapAction'      : 'Load custom colour map',
-    'SavePerspectiveAction'    : 'Save current perspective',
-    'ClearPerspectiveAction'   : 'Clear all perspectives',
+    'SaveLayoutAction'         : 'Save current layout',
+    'ClearLayoutsAction'       : 'Clear all layouts',
     'DiagnosticReportAction'   : 'Diagnostic report',
     'RunScriptAction'          : 'Run script',
     'AboutAction'              : 'About FSLeyes',
     'PearsonCorrelateAction'   : 'Seed correlation (Pearson)',
-    'ApplyFlirtXfmAction'      : 'Load FLIRT transformation',
-    'SaveFlirtXfmAction'       : 'Export FLIRT transformation',
+    'ApplyFlirtXfmAction'      : 'Load affine transformation',
+    'SaveFlirtXfmAction'       : 'Export affine transformation',
+    'NotebookAction'           : 'Open notebooks',
     'PCACorrelateAction'       : 'Seed correlation (PCA)',
     'ResampleAction'           : 'Resample image',
 
@@ -440,6 +479,7 @@ actions = TypeDict({
     'ViewPanel.removeAllPanels'             : 'Remove all panels',
 
     'CanvasPanel.screenshot'                : 'Take screenshot',
+    'CanvasPanel.movieGif'                  : 'Save animated GIF',
     'CanvasPanel.showCommandLineArgs'       : 'Show command line for scene',
     'CanvasPanel.applyCommandLineArgs'      : 'Apply command line arguments',
     'CanvasPanel.toggleMovieMode'           : 'Movie mode',
@@ -474,7 +514,8 @@ actions = TypeDict({
 
     'LightBoxPanel.toggleLightBoxToolBar' : 'Lightbox toolbar',
 
-    'Scene3DPanel.resetDisplay' : 'Reset camera',
+    'Scene3DPanel.resetDisplay'         : 'Reset camera',
+    'Scene3DPanel.toggleScene3DToolBar' : '3D toolbar',
 
     'PlotPanel.screenshot'                          : 'Take screenshot',
     'PlotPanel.importDataSeries'                    : 'Import ...',
@@ -502,6 +543,7 @@ actions = TypeDict({
     'OrthoEditProfile.createMask'              : 'Create mask',
     'OrthoEditProfile.clearSelection'          : 'Clear selection',
     'OrthoEditProfile.fillSelection'           : 'Fill selection',
+    'OrthoEditProfile.invertSelection'         : 'Invert selection',
     'OrthoEditProfile.eraseSelection'          : 'Erase selection',
     'OrthoEditProfile.copySelection'           : 'Copy selection',
     'OrthoEditProfile.pasteSelection'          : 'Paste selection',
@@ -514,13 +556,20 @@ labels = TypeDict({
     'FSLeyesFrame.noName'                 : '<unnamed>',
     'FSLeyesFrame.recentPathsMenu'        : 'Recent files',
 
-    'LocationPanel.worldLocation'         : 'Coordinates: ',
-    'LocationPanel.worldLocation.unknown' : 'Unknown',
-    'LocationPanel.voxelLocation'         : 'Voxel location',
-    'LocationPanel.volume'                : 'Volume',
-    'LocationPanel.noData'                : 'No data',
-    'LocationPanel.outOfBounds'           : 'Out of bounds',
-    'LocationPanel.notAvailable'          : 'N/A',
+    'LocationPanel.info'                      : 'Location',
+    'LocationPanel.history'                   : 'History',
+    'LocationInfoPanel.worldLocation'         : 'Coordinates: ',
+    'LocationInfoPanel.worldLocation.unknown' : 'Unknown',
+    'LocationInfoPanel.voxelLocation'         : 'Voxel location',
+    'LocationInfoPanel.volume'                : 'Volume',
+    'LocationInfoPanel.noData'                : 'No data',
+    'LocationInfoPanel.outOfBounds'           : 'Out of bounds',
+    'LocationInfoPanel.notAvailable'          : 'N/A',
+    'LocationHistoryPanel.load'               : 'Load',
+    'LocationHistoryPanel.save'               : 'Save',
+    'LocationHistoryPanel.clear'              : 'Clear',
+    'LocationHistoryPanel.hint'               :
+    'Double click on an item to add a comment.',
 
     'OverlayListPanel.noDataSource'       : '[in memory]',
 
@@ -606,6 +655,7 @@ labels = TypeDict({
 
     'OverlayDisplayPanel.loadCmap'       : 'Load colour map',
     'OverlayDisplayPanel.loadVertexData' : 'Load data',
+    'OverlayDisplayPanel.loadVertices'   : 'Load vertices',
 
     'OverlayDisplayPanel.clipPlane#'     : 'Clip plane #{}',
 
@@ -621,6 +671,7 @@ labels = TypeDict({
     'OverlayInfoPanel.Nifti.dimensions'   : 'Dimensions',
     'OverlayInfoPanel.Nifti.transform'    : 'Transform/space',
     'OverlayInfoPanel.Nifti.orient'       : 'Orientation',
+    'OverlayInfoPanel.Nifti.size'         : 'Size',
 
     'OverlayInfoPanel.Nifti.displaySpace.id'          : 'Raw voxels',
     'OverlayInfoPanel.Nifti.displaySpace.pixdim'      : 'True scaled voxels',
@@ -641,30 +692,36 @@ labels = TypeDict({
                                                   '(MELODIC analysis)',
     'OverlayInfoPanel.MelodicImage.melodicInfo' : 'MELODIC information',
 
-    'OverlayInfoPanel.TriangleMesh'                        :
-    'VTK model',
-    'OverlayInfoPanel.TriangleMesh.numVertices'            :
+    'OverlayInfoPanel.Mesh'            :
+    '3D mesh',
+    'OverlayInfoPanel.Mesh.numVertices'            :
     'Number of vertices',
-    'OverlayInfoPanel.TriangleMesh.numTriangles'           :
+    'OverlayInfoPanel.Mesh.numTriangles'           :
     'Number of triangles',
-    'OverlayInfoPanel.TriangleMesh.displaySpace'           :
+    'OverlayInfoPanel.Mesh.displaySpace'           :
     'Display space',
-    'OverlayInfoPanel.TriangleMesh.refImage'               :
+    'OverlayInfoPanel.Mesh.refImage'               :
     'Reference image',
-    'OverlayInfoPanel.TriangleMesh.coordSpace'             :
+    'OverlayInfoPanel.Mesh.size'                   :
+    'Size',
+    'OverlayInfoPanel.Mesh.coordSpace'             :
     'Vertices defined in',
-    'OverlayInfoPanel.TriangleMesh.coordSpace.id'          :
+    'OverlayInfoPanel.Mesh.coordSpace.id'          :
     'Voxels ({})',
-    'OverlayInfoPanel.TriangleMesh.coordSpace.pixdim'      :
+    'OverlayInfoPanel.Mesh.coordSpace.pixdim'      :
     'Scaled voxels ({})',
-    'OverlayInfoPanel.TriangleMesh.coordSpace.pixdim-flip' :
+    'OverlayInfoPanel.Mesh.coordSpace.pixdim-flip' :
     'Scaled voxels [FSL convention] ({})',
-    'OverlayInfoPanel.TriangleMesh.coordSpace.affine'      :
+    'OverlayInfoPanel.Mesh.coordSpace.affine'      :
     'World coordinates ({})',
-    'OverlayInfoPanel.TriangleMesh.coordSpace.display'     :
+    'OverlayInfoPanel.Mesh.coordSpace.display'     :
     'Display coordinate system',
+    'OverlayInfoPanel.Mesh.coordSpace.torig'      :
+    'Freesurfer coordinates',
 
-    'OverlayInfoPanel.GiftiSurface' : 'GIFTI surface',
+    'OverlayInfoPanel.VTKMesh'        : 'VTK model',
+    'OverlayInfoPanel.GiftiMesh'      : 'GIFTI surface',
+    'OverlayInfoPanel.FreesurferMesh' : 'Freesurfer surface',
 
     'OverlayInfoPanel.dataSource'               : 'Data source',
     'OverlayInfoPanel.niftiVersion'             : 'NIFTI version',
@@ -672,6 +729,13 @@ labels = TypeDict({
     'OverlayInfoPanel.DTIFitTensor'             : 'DTIFit tensor images',
     'OverlayInfoPanel.DTIFitTensor.tensorInfo'  : 'Tensor image paths ',
 
+    'OverlayInfoPanel.DicomImage'               : 'NIFTI Image (from DICOM)',
+    'OverlayInfoPanel.DicomImage.dicomDir'      : 'DICOM directory',
+    'OverlayInfoPanel.DicomImage.dicomInfo'     : 'DICOM information',
+
+    'OverlayInfoPanel.MGHImage'                 : 'NIFTI Image (from MGH)',
+    'OverlayInfoPanel.MGHImage.filename'        : 'MGH image file',
+
     'MelodicClassificationPanel.componentTab'   : 'Components',
     'MelodicClassificationPanel.labelTab'       : 'Labels',
     'MelodicClassificationPanel.loadButton'     : 'Load labels',
@@ -707,11 +771,10 @@ labels = TypeDict({
     'CropImagePanel.save'             : 'Save',
     'CropImagePanel.cancel'           : 'Cancel',
 
-    'EditTransformPanel.dsWarning'   :
+    'EditTransformPanel.dsWarning'  :
     'Warning: Change the display space to "World" to see\n'
     'the effects of the transformation. You can change it\n'
     'back in the view settings panel',
-    'EditTransformPanel.changeDS'    : 'Change display space',
     'EditTransformPanel.noOverlay'   : 'Select a NIFTI image',
     'EditTransformPanel.overlayName' : 'Transform {}',
     'EditTransformPanel.oldXform'    : 'Original transform',
@@ -719,25 +782,34 @@ labels = TypeDict({
     'EditTransformPanel.scale'       : 'Scale',
     'EditTransformPanel.offset'      : 'Translate',
     'EditTransformPanel.rotate'      : 'Rotate',
+    'EditTransformPanel.centre'      : 'Centre of rotation',
     'EditTransformPanel.apply'       : 'Apply',
     'EditTransformPanel.reset'       : 'Reset',
-    'EditTransformPanel.loadFlirt'   : 'Load FLIRT',
-    'EditTransformPanel.saveFlirt'   : 'Save FLIRT',
+    'EditTransformPanel.loadFlirt'   : 'Load affine',
+    'EditTransformPanel.saveFlirt'   : 'Save affine',
     'EditTransformPanel.cancel'      : 'Close',
 
+    'EditTransformPanel.centre.options' : {
+        'volume' : 'Rotate around centre of image volume',
+        'cursor' : 'Rotate around current cursor location'
+    },
+
+    'DisplaySpaceWarning.changeDS'    : 'Change display space',
 
     'FlirtFileDialog.load.message' :
-    'Select a FLIRT transformation matrix\n'
-    'and the corresponding reference image.',
+    'Select an affine transformation matrix file',
 
     'FlirtFileDialog.save.message' :
-    'Specify a file name for the FLIRT transformation\n'
-    'matrix, and choose the corresponding reference image.',
+    'Specify a file name for the affine transformation',
 
     'FlirtFileDialog.source'              : 'Source image:\n{}',
     'FlirtFileDialog.refChoiceSelectFile' : 'Select file manually',
     'FlirtFileDialog.matFile'             : 'Matrix file',
-    'FlirtFileDialog.refFile'             : 'Reference image',
+    'FlirtFileDialog.affType'             : 'File type',
+    'FlirtFileDialog.affType.flirt'       : 'FLIRT',
+    'FlirtFileDialog.affType.v2w'         : 'Voxel-to-world',
+    'FlirtFileDialog.refImage'            : 'Reference image',
+    'FlirtFileDialog.refFile'             : 'Reference image file',
     'FlirtFileDialog.selectFile'          : 'Choose',
     'FlirtFileDialog.ok'                  : 'Ok',
     'FlirtFileDialog.cancel'              : 'Cancel',
@@ -760,6 +832,22 @@ labels = TypeDict({
     'ResampleDialog.origPixdims'   : 'Old pixdims',
     'ResampleDialog.newVoxels'     : 'New shape',
     'ResampleDialog.newPixdims'    : 'New pixdims',
+
+    'XNATBrowser.ok'        : 'Download',
+    'XNATBrowser.cancel'    : 'Cancel',
+    'XNATBrowser.choosedir' : 'Choose a download location',
+
+    'BrowseDicomPanel.dicomdir'          : 'Directory',
+    'BrowseDicomPanel.date'              : 'Date',
+    'BrowseDicomPanel.patient'           : 'Patient',
+    'BrowseDicomPanel.institution'       : 'Institution',
+    'BrowseDicomPanel.SeriesNumber'      : 'Series',
+    'BrowseDicomPanel.SeriesDescription' : 'Description',
+    'BrowseDicomPanel.Matrix'            : 'Matrix',
+    'BrowseDicomPanel.Load'              : 'Load',
+
+    'BrowseDicomDialog.load'       : 'Load',
+    'BrowseDicomDialog.cancel'     : 'Cancel',
 })
 
 
@@ -771,6 +859,7 @@ properties = TypeDict({
     'CanvasPanel.syncLocation'       : 'Link location',
     'CanvasPanel.syncOverlayOrder'   : 'Link overlay order',
     'CanvasPanel.syncOverlayDisplay' : 'Link overlay display settings',
+    'CanvasPanel.syncOverlayVolume'  : 'Link overlay volume settings',
     'CanvasPanel.movieMode'          : 'Movie mode',
     'CanvasPanel.movieRate'          : 'Movie update rate',
     'CanvasPanel.movieAxis'          : 'Movie axis',
@@ -784,6 +873,7 @@ properties = TypeDict({
     'SceneOpts.showColourBar'      : 'Show colour bar',
     'SceneOpts.performance'        : 'Rendering performance',
     'SceneOpts.zoom'               : 'Zoom',
+    'SceneOpts.highDpi'            : 'Enable high-DPI rendering',
     'SceneOpts.colourBarLocation'  : 'Colour bar location',
     'SceneOpts.colourBarLabelSide' : 'Colour bar label side',
 
@@ -829,6 +919,7 @@ properties = TypeDict({
     'TimeSeriesPanel.plotResiduals'    : 'Plot residuals',
 
     'HistogramPanel.histType'    : 'Histogram type',
+    'HistogramPanel.plotType'    : 'Plot type',
 
     'PowerSpectrumPanel.plotFrequencies' : 'Show frequencies along x axis ',
     'PowerSpectrumPanel.plotMelodicICs'  : 'Plot component power spectra for '
@@ -890,6 +981,7 @@ properties = TypeDict({
     'ColourMapOpts.cmap'             : 'Colour map',
     'ColourMapOpts.custom_cmap'      : 'Colour map',
     'ColourMapOpts.cmapResolution'   : 'Colour map resolution',
+    'ColourMapOpts.gamma'            : 'Gamma correction',
     'ColourMapOpts.interpolateCmaps' : 'Interpolate colour maps',
     'ColourMapOpts.negativeCmap'     : '-ve colour map',
     'ColourMapOpts.useNegativeCmap'  : '-ve colour map',
@@ -902,19 +994,23 @@ properties = TypeDict({
     'VolumeOpts.overrideDataRange'        : 'Override image data range',
     'VolumeOpts.custom_overrideDataRange' : 'Override image data range',
 
-    'Volume3DOpts.dithering'               : 'Dithering',
     'Volume3DOpts.numSteps'                : 'Number of samples',
     'Volume3DOpts.blendFactor'             : 'Blending',
+    'Volume3DOpts.smoothing'               : 'Smoothing',
     'Volume3DOpts.resolution'              : 'Quality',
     'Volume3DOpts.numClipPlanes'           : 'Number of clipping planes',
     'Volume3DOpts.showClipPlanes'          : 'Show clipping planes',
+    'Volume3DOpts.clipMode'                : 'Clipping mode',
     'Volume3DOpts.clipPosition'            : 'Clip position (%)',
     'Volume3DOpts.clipInclination'         : 'Clip Z angle',
     'Volume3DOpts.clipAzimuth'             : 'Clip rotation',
 
-    'MaskOpts.colour'         : 'Colour',
-    'MaskOpts.invert'         : 'Invert',
-    'MaskOpts.threshold'      : 'Threshold',
+    'MaskOpts.colour'        : 'Colour',
+    'MaskOpts.invert'        : 'Invert',
+    'MaskOpts.threshold'     : 'Threshold',
+    'MaskOpts.outline'       : 'Show outline only',
+    'MaskOpts.outlineWidth'  : 'Outline width',
+    'MaskOpts.interpolation' : 'Interpolation',
 
     'VectorOpts.xColour'       : 'X Colour',
     'VectorOpts.yColour'       : 'Y Colour',
@@ -944,7 +1040,9 @@ properties = TypeDict({
     'MeshOpts.outlineWidth'      : 'Outline width',
     'MeshOpts.refImage'          : 'Reference image',
     'MeshOpts.coordSpace'        : 'Mesh coordinate space',
+    'MeshOpts.custom_vertexSet'  : 'Surface definition',
     'MeshOpts.custom_vertexData' : 'Vertex data',
+    'MeshOpts.vertexSet'         : 'Surface definition',
     'MeshOpts.vertexData'        : 'Vertex data',
     'MeshOpts.vertexDataIndex'   : 'Vertex data index',
     'MeshOpts.showName'          : 'Show model name',
@@ -964,6 +1062,7 @@ properties = TypeDict({
     'TensorOpts.tensorScale'       : 'Tensor size',
 
     'SHOpts.lighting'        : 'Lighting effects',
+    'SHOpts.normalise'       : 'Normalise FOD sizes',
     'SHOpts.size'            : 'FOD size',
     'SHOpts.radiusThreshold' : 'Radius threshold',
     'SHOpts.shResolution'    : 'FOD quality',
@@ -1039,7 +1138,8 @@ choices = TypeDict({
 
     'MeshOpts.refImage.none'     : 'No reference image',
 
-    'MeshOpts.coordSpace' : {'affine'      : 'World coordinates',
+    'MeshOpts.coordSpace' : {'torig'       : 'Freesurfer coordinates',
+                             'affine'      : 'World coordinates',
                              'pixdim'      : 'Scaled voxels',
                              'pixdim-flip' : 'Scaled voxels forced to '
                                              'radiological orientation',
@@ -1058,22 +1158,27 @@ choices = TypeDict({
                                   'linear' : 'Linear interpolation',
                                   'spline' : 'Spline interpolation'},
 
+    'Volume3DOpts.clipMode' : {'intersection' : 'Intersection',
+                               'union'        : 'Union',
+                               'complement'   : 'Complement'},
 
     'SHOpts.colourMode' : {'radius'    : 'Colour by radius',
                            'direction' : 'Colour by direction'},
 
-    'Display.overlayType' : {'volume'     : '3D/4D volume',
-                             'mask'       : '3D/4D mask image',
-                             'label'      : 'Label image',
-                             'rgbvector'  : '3-direction vector image (RGB)',
-                             'linevector' : '3-direction vector image (Line)',
-                             'mesh'       : '3D mesh',
-                             'giftimesh'  : 'GIFTI surface',
-                             'tensor'     : 'Diffusion tensor',
-                             'sh'         : 'Diffusion SH'},
+    'Display.overlayType' : {
+        'volume'         : '3D/4D volume',
+        'mask'           : '3D/4D mask image',
+        'label'          : 'Label image',
+        'rgbvector'      : '3-direction vector image (RGB)',
+        'linevector'     : '3-direction vector image (Line)',
+        'mesh'           : '3D mesh',
+        'tensor'         : 'Diffusion tensor',
+        'sh'             : 'Diffusion SH'},
 
     'HistogramPanel.histType' : {'probability' : 'Probability',
                                  'count'       : 'Count'},
+    'HistogramPanel.plotType' : {'centre'      : 'Bin centres',
+                                 'edge'        : 'Bin edges'},
 
     'DataSeries.lineStyle' : {'-'  : 'Solid line',
                               '--' : 'Dashed line',
@@ -1291,14 +1396,16 @@ melodic = TypeDict({
     'report'         : 'Link to report',
 })
 
-perspectives = {
+layouts = {
     'default'  : 'Default layout',
     'melodic'  : 'MELODIC mode',
     'feat'     : 'FEAT mode',
     'ortho'    : 'Plain orthographic',
     'lightbox' : 'Plain lightbox',
+    '3d'       : 'Plain 3D',
 }
 
+
 tensor = {
     'v1' : 'First eigenvector image',
     'v2' : 'Second eigenvector image',
@@ -1342,26 +1449,34 @@ about = {
 
     FSLeyes is a Python application which leverages the following open-source software libraries:
 
+     - fsleyes-props [{}] (https://git.fmrib.ox.ac.uk/fsl/fsleyes/props)
+     - fsleyes-widgets [{}] (https://git.fmrib.ox.ac.uk/fsl/fsleyes/widgets)
+     - fslpy [{}] (https://git.fmrib.ox.ac.uk/fsl/fslpy)
      - indexed_gzip [{}] (https://github.com/pauldmccarthy/indexed_gzip/)
+     - IPython [{}] (https://ipython.org/)
      - jinja2 [{}] (http://jinja.pocoo.org)
+     - Jupyter notebook [{}] (https://jupyter.org)
      - matplotlib [{}] (http://www.matplotlib.org)
      - nibabel [{}] (http://nipy.org/nibabel)
      - numpy [{}] (http://www.numpy.org)
      - pillow [{}]  (http://python-pillow.org/)
-     - fsleyes-props [{}] (https://git.fmrib.ox.ac.uk/fsl/fsleyes/props)
-     - fsleyes-widgets [{}] (https://git.fmrib.ox.ac.uk/fsl/fsleyes/widgets)
-     - fslpy [{}] (https://git.fmrib.ox.ac.uk/fsl/fslpy)
      - pyopengl [{}] (http://pyopengl.sourceforge.net)
      - pyparsing [{}] (http://pyparsing.wikispaces.com/)
      - scipy [{}] (http://www.scipy.org)
      - six [{}] (https://pythonhosted.org/six/)
      - trimesh [{}] (https://github.com/mikedh/trimesh)
-     - wxPython [{}] (http://www.wxpython.org)
+     - wxpython [{}] (http://www.wxpython.org)
+     - wxnatpy [{}] (https://github.com/pauldmccarthy/wxnatpy/)
+     - xnatpy [{}] (https://bitbucket.org/bigr_erasmusmc/xnatpy)
 
     Cubic/spline interpolation routines used in FSLeyes are provided by Daniel Ruijters and Philippe Th\u00E9venaz, described at http://www.dannyruijters.nl/cubicinterpolation/.
 
+    The GLSL parser is based on code by Nicolas P . Rougier, available at https://github.com/rougier/glsl-parser, and released under the BSD license.
+
     Some of the icons used in FSLeyes are derived from the Freeline icon set, by Enes Dal, available at https://www.iconfinder.com/Enesdal, and released under the Creative Commons (Attribution 3.0 Unported) license.
 
+    DICOM to NIFTI conversion is performed with Chris Rorden's dcm2niix (https://github.com/rordenlab/dcm2niix).
+
     FSLeyes is released under Version 2.0 of the Apache Software License. Source code for FSLeyes is available at https://git.fmrib.ox.ac.uk/fsl/fsleyes/fsleyes
 
     Copyright 2016-2017 University of Oxford, Oxford, UK.
@@ -1372,9 +1487,11 @@ about = {
     # dynamically looks up the version number for
     # each of them, and inserts them into the above
     # string.
-    'libs' : ['indexed_gzip',  'jinja2',             'matplotlib',
-              'nibabel',       'numpy',              'PIL',
-              'fsleyes_props', 'fsleyes_widgets',    'fsl.version',
-              'OpenGL',        'pyparsing',          'scipy',
-              'six',           'fsleyes.gl.trimesh', 'wx'],
+    'libs' : ['fsleyes_props', 'fsleyes_widgets',    'fsl.version',
+              'indexed_gzip',  'IPython',            'jinja2',
+              'notebook',      'matplotlib',         'nibabel',
+              'numpy',         'PIL',                'OpenGL',
+              'pyparsing',     'scipy',              'six',
+              'trimesh',       'wx',                 'wxnat',
+              'xnat'],
 }
