@@ -80,7 +80,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
     The display of an ``OrthoPanel`` can be configured through all of the
     settings provided by the :class:`.OrthoOpts` class. The ``OrthoOpts``
     instance for a given ``OrthoPanel`` can be accessed via the
-    :meth:`.CanvasPanel.getSceneOptions` method.
+    :meth:`.CanvasPanel.sceneOpts` method.
 
 
     **Interaction**
@@ -144,8 +144,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                                          frame,
                                          sceneOpts)
 
-        name         = self.getName()
-        contentPanel = self.getContentPanel()
+        name         = self.name
+        contentPanel = self.contentPanel
 
         # The canvases themselves - each one displays a
         # slice along each of the three world axes
@@ -207,6 +207,9 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         yopts.bindProps('renderMode',   sceneOpts)
         zopts.bindProps('renderMode',   sceneOpts)
 
+        xopts.bindProps('highDpi',      sceneOpts)
+        yopts.bindProps('highDpi',      sceneOpts)
+        zopts.bindProps('highDpi',      sceneOpts)
 
         # Callbacks for ortho panel layout options
         sceneOpts.addListener('layout', name, self.__refreshLayout)
@@ -252,8 +255,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         from fsleyes.actions.correlate import PearsonCorrelateAction
 
         self.__pCorrAction = PearsonCorrelateAction(
-            self.getOverlayList(),
-            self.getDisplayContext(),
+            self.overlayList,
+            self.displayCtx,
             self)
 
         self.pearsonCorrelation.bindProps('enabled', self.__pCorrAction)
@@ -280,21 +283,21 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         :class:`.SliceCanvas` panels, and calls :meth:`.CanvasPanel.destroy`.
         """
 
-        sceneOpts    = self.getSceneOptions()
-        contentPanel = self.getContentPanel()
+        sceneOpts    = self.sceneOpts
+        contentPanel = self.contentPanel
 
-        sceneOpts        .removeListener('showXCanvas',      self._name)
-        sceneOpts        .removeListener('showYCanvas',      self._name)
-        sceneOpts        .removeListener('showZCanvas',      self._name)
-        sceneOpts        .removeListener('labelSize',        self._name)
-        sceneOpts        .removeListener('fgColour',         self._name)
-        sceneOpts        .removeListener('showLabels',       self._name)
-        self._displayCtx .removeListener('location',         self._name)
-        self._displayCtx .removeListener('bounds',           self._name)
-        self._displayCtx .removeListener('selectedOverlay',  self._name)
-        self._displayCtx .removeListener('displaySpace',     self._name)
-        self._displayCtx .removeListener('radioOrientation', self._name)
-        self._overlayList.removeListener('overlays',         self._name)
+        sceneOpts       .removeListener('showXCanvas',      self.name)
+        sceneOpts       .removeListener('showYCanvas',      self.name)
+        sceneOpts       .removeListener('showZCanvas',      self.name)
+        sceneOpts       .removeListener('labelSize',        self.name)
+        sceneOpts       .removeListener('fgColour',         self.name)
+        sceneOpts       .removeListener('showLabels',       self.name)
+        self.displayCtx .removeListener('location',         self.name)
+        self.displayCtx .removeListener('bounds',           self.name)
+        self.displayCtx .removeListener('selectedOverlay',  self.name)
+        self.displayCtx .removeListener('displaySpace',     self.name)
+        self.displayCtx .removeListener('radioOrientation', self.name)
+        self.overlayList.removeListener('overlays',         self.name)
 
         self.__labelMgr.destroy()
         self.__xcanvas.destroy()
@@ -434,6 +437,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         :mod:`.actions` that are defined on this ``OrthoPanel``.
         """
         actionz = [self.screenshot,
+                   self.movieGif,
                    self.showCommandLineArgs,
                    self.applyCommandLineArgs,
                    None,
@@ -566,7 +570,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
 
         # It's unlikely, but an OrthoPanel might be
         # created without a ref to a FSLeyesFrame.
-        if self.getFrame() is not None:
+        if self.frame is not None:
             if inEdit: self.__addEditMenu()
             else:      self.__removeEditMenu()
 
@@ -577,9 +581,9 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         menu to the :class:`.FSLeyesFrame`.
         """
 
-        frame    = self.getFrame()
+        frame    = self.frame
         menuName = strings.labels[self, 'editMenu']
-        menuName = menuName.format(self.getFrame().getViewPanelID(self))
+        menuName = menuName.format(frame.getViewPanelID(self))
         menuBar  = frame.GetMenuBar()
         profile  = self.getCurrentProfile()
         idx      = menuBar.FindMenu(menuName)
@@ -603,7 +607,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                    'fillSelection',
                    'eraseSelection',
                    'copySelection',
-                   'pasteSelection']
+                   'pasteSelection',
+                   'invertSelection']
 
         frame.populateMenu(editMenu,
                            profile,
@@ -631,7 +636,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         if self.__editMenuTitle is None:
             return
 
-        frame        = self.getFrame()
+        frame        = self.frame
         editMenuName = self.__editMenuTitle
         menuBar      = frame.GetMenuBar()
         idx          = menuBar.FindMenu(editMenuName)
@@ -655,7 +660,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         Shows/hides each of the :class:`.SliceCanvas` panels accordingly.
         """
 
-        opts     = self.getSceneOptions()
+        opts     = self.sceneOpts
         canvases = [self.__xcanvas,   self.__ycanvas,   self.__zcanvas]
         shows    = [opts.showXCanvas, opts.showYCanvas, opts.showZCanvas]
 
@@ -682,11 +687,11 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         necessary.
         """
 
-        if len(self._overlayList) == 0:
+        if len(self.overlayList) == 0:
             return
 
-        inRadio = self._displayCtx.displaySpaceIsRadiological()
-        flip    = self._displayCtx.radioOrientation != inRadio
+        inRadio = self.displayCtx.displaySpaceIsRadiological()
+        flip    = self.displayCtx.radioOrientation != inRadio
 
         self.__ycanvas.opts.invertX = flip
         self.__zcanvas.opts.invertX = flip
@@ -699,12 +704,12 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         """
 
         # Disable actions that need an overlay
-        haveOverlays = len(self._overlayList) > 0
-        selOverlay   = self._displayCtx.getSelectedOverlay()
+        haveOverlays = len(self.overlayList) > 0
+        selOverlay   = self.displayCtx.getSelectedOverlay()
 
         if selOverlay is not None:
 
-            display = self._displayCtx.getDisplay(selOverlay)
+            display = self.displayCtx.getDisplay(selOverlay)
             isImage = isinstance(selOverlay, fslimage.Image) and \
                       display.overlayType in ('volume', 'mask', 'label')
         else:
@@ -736,17 +741,17 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         calculation.
         """
 
-        opts   = self.getSceneOptions()
+        opts   = self.sceneOpts
         layout = opts.layout
 
-        width, height = self.getContentPanel().GetClientSize().Get()
+        width, height = self.contentPanel.GetClientSize().Get()
 
         show     = [opts.showXCanvas,  opts.showYCanvas,  opts.showZCanvas]
         canvases = [self.__xcanvas,    self.__ycanvas,    self.__zcanvas]
 
-        if width == 0 or height == 0:   return
-        if len(self._overlayList) == 0: return
-        if not any(show):               return
+        if width == 0 or height == 0:  return
+        if len(self.overlayList) == 0: return
+        if not any(show):              return
 
         canvases = [c for (c, s) in zip(canvases, show) if s]
 
@@ -766,9 +771,9 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         # fsllayout) provides functions to do
         # this for us
         canvasaxes = [(c.opts.xax, c.opts.yax) for c in canvases]
-        axisLens   = [self._displayCtx.bounds.xlen,
-                      self._displayCtx.bounds.ylen,
-                      self._displayCtx.bounds.zlen]
+        axisLens   = [self.displayCtx.bounds.xlen,
+                      self.displayCtx.bounds.ylen,
+                      self.displayCtx.bounds.zlen]
 
         sizes = fsllayout.calcSizes(layout,
                                     canvasaxes,
@@ -791,7 +796,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
 
         refresh = kwa.pop('refresh', True)
 
-        opts   = self.getSceneOptions()
+        opts   = self.sceneOpts
         layout = opts.layout
 
         # We lay out all canvases, even
@@ -845,7 +850,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         for c in canvases:
             self.__canvasSizer.Add(c, flag=flag | wx.EXPAND)
 
-        self.getContentPanel().SetSizer(self.__canvasSizer)
+        self.contentPanel.SetSizer(self.__canvasSizer)
 
         # Calculate/ adjust the appropriate sizes
         # for each canvas, such that they are scaled
@@ -861,7 +866,7 @@ class OrthoPanel(canvaspanel.CanvasPanel):
 
         if refresh:
             self.Layout()
-            self.getContentPanel().Layout()
+            self.contentPanel.Layout()
             self.Refresh()
 
 
